using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using GameEngine.Helpers;
using GameEngine;
using Microsoft.Xna.Framework;
using GameEngine.Xml;

namespace Editor
{
    public class HelperDrawer
    {
        Effect primitiveEffect;
        GraphicsDevice graphicsDevice;

        private VertexBuffer axisBuffer;
        private VertexBuffer pointBuffer;
        private VertexBuffer obbBuffer;
        private VertexDeclaration vpcDecl;


        int obbLineCount = 12;
        int axisLineCount = 3;
        int pointLineCount = 8;

        public HelperDrawer(Effect primitivesEffect, GraphicsDevice graphicsDevice)
        {
            this.primitiveEffect = primitivesEffect;
            this.graphicsDevice = graphicsDevice;

            InitializeOBBDrawer();
            InitializeAxisGismoDrawer();
            InitializePointDrawer();
            vpcDecl = new VertexDeclaration(graphicsDevice, VertexPositionColor.VertexElements);
        }

        private void InitializeOBBDrawer()
        {

            obbBuffer = new VertexBuffer(graphicsDevice, typeof(VertexPositionColor), obbLineCount * 2, BufferUsage.WriteOnly);

            VertexPositionColor[] vertices = new VertexPositionColor[obbLineCount * 2];

            int i = 0;

            vertices[i++] = new VertexPositionColor(new Vector3(1, 1, 1), Color.White);
            vertices[i++] = new VertexPositionColor(new Vector3(1, -1, 1), Color.White);

            vertices[i++] = new VertexPositionColor(new Vector3(1, -1, 1), Color.White);
            vertices[i++] = new VertexPositionColor(new Vector3(1, -1, -1), Color.White);

            vertices[i++] = new VertexPositionColor(new Vector3(1, -1, -1), Color.White);
            vertices[i++] = new VertexPositionColor(new Vector3(1, 1, -1), Color.White);

            vertices[i++] = new VertexPositionColor(new Vector3(1, 1, -1), Color.White);
            vertices[i++] = new VertexPositionColor(new Vector3(1, 1, 1), Color.White);


            vertices[i++] = new VertexPositionColor(new Vector3(-1, 1, 1), Color.White);
            vertices[i++] = new VertexPositionColor(new Vector3(-1, -1, 1), Color.White);

            vertices[i++] = new VertexPositionColor(new Vector3(-1, -1, 1), Color.White);
            vertices[i++] = new VertexPositionColor(new Vector3(-1, -1, -1), Color.White);

            vertices[i++] = new VertexPositionColor(new Vector3(-1, -1, -1), Color.White);
            vertices[i++] = new VertexPositionColor(new Vector3(-1, 1, -1), Color.White);

            vertices[i++] = new VertexPositionColor(new Vector3(-1, 1, -1), Color.White);
            vertices[i++] = new VertexPositionColor(new Vector3(-1, 1, 1), Color.White);


            vertices[i++] = new VertexPositionColor(new Vector3(1, 1, 1), Color.White);
            vertices[i++] = new VertexPositionColor(new Vector3(-1, 1, 1), Color.White);

            vertices[i++] = new VertexPositionColor(new Vector3(1, 1, -1), Color.White);
            vertices[i++] = new VertexPositionColor(new Vector3(-1, 1, -1), Color.White);

            vertices[i++] = new VertexPositionColor(new Vector3(1, -1, 1), Color.White);
            vertices[i++] = new VertexPositionColor(new Vector3(-1, -1, 1), Color.White);

            vertices[i++] = new VertexPositionColor(new Vector3(1, -1, -1), Color.White);
            vertices[i++] = new VertexPositionColor(new Vector3(-1, -1, -1), Color.White);

            obbBuffer.SetData(vertices);
        }

        private void InitializeAxisGismoDrawer()
        {
            axisBuffer = new VertexBuffer(graphicsDevice, typeof(VertexPositionColor), axisLineCount * 2, BufferUsage.WriteOnly);

            VertexPositionColor[] vertices = new VertexPositionColor[axisLineCount * 2];

            int i = 0;

            vertices[i++] = new VertexPositionColor(new Vector3(0.25f, 0, 0), Color.Red);
            vertices[i++] = new VertexPositionColor(Vector3.Zero, Color.Red);

            vertices[i++] = new VertexPositionColor(new Vector3(0, 0.25f, 0), Color.Green);
            vertices[i++] = new VertexPositionColor(Vector3.Zero, Color.Green);

            vertices[i++] = new VertexPositionColor(new Vector3(0, 0, 0.25f), Color.Blue);
            vertices[i++] = new VertexPositionColor(Vector3.Zero, Color.Blue);

            axisBuffer.SetData(vertices);
        }

        private void InitializePointDrawer()
        {
            pointBuffer = new VertexBuffer(graphicsDevice, typeof(VertexPositionColor), pointLineCount * 2, BufferUsage.WriteOnly);

            VertexPositionColor[] vertices = new VertexPositionColor[pointLineCount * 2];

            int i = 0;

            Vector3 startPoint = Vector3.One;

            vertices[i++] = new VertexPositionColor(startPoint, Color.White);
            vertices[i++] = new VertexPositionColor(startPoint * -1, Color.White);

            startPoint.X = -1;

            vertices[i++] = new VertexPositionColor(startPoint, Color.White);
            vertices[i++] = new VertexPositionColor(startPoint * -1, Color.White);

            startPoint.Y = -1;

            vertices[i++] = new VertexPositionColor(startPoint, Color.White);
            vertices[i++] = new VertexPositionColor(startPoint * -1, Color.White);

            startPoint.X = 1;

            vertices[i++] = new VertexPositionColor(startPoint, Color.White);
            vertices[i++] = new VertexPositionColor(startPoint * -1, Color.White);

            pointBuffer.SetData(vertices);
        }

        public void DrawPoint(Vector3 position, DrawInfo drawInfo, Color color)
        {
            DrawPoint(position, 1, drawInfo, color);
        }

        public void DrawPoint(Vector3 position, float scale, DrawInfo drawInfo, Color color)
        {
            Matrix world = Matrix.CreateScale(scale) * Matrix.CreateTranslation(position);

            drawInfo.GraphicsDevice.VertexDeclaration = vpcDecl;
            drawInfo.GraphicsDevice.Vertices[0].SetSource(pointBuffer, 0, VertexPositionColor.SizeInBytes);
            primitiveEffect.Parameters["WorldViewProj"].SetValue((world * drawInfo.Camera.View) * drawInfo.Camera.Projection);
            primitiveEffect.Parameters["Color"].SetValue(color.ToVector4());
            primitiveEffect.Begin();

            foreach (EffectPass pass in primitiveEffect.CurrentTechnique.Passes)
            {
                pass.Begin();
                drawInfo.GraphicsDevice.DrawPrimitives(PrimitiveType.LineList, 0, pointLineCount);
                pass.End();
            }

            primitiveEffect.End();
        }

        public void DrawAxisGismo(Matrix rotation, Vector3 scale, Vector3 position, DrawInfo drawInfo)
        {
            Matrix world = Matrix.CreateScale(scale) * rotation * Matrix.CreateTranslation(position);

            drawInfo.GraphicsDevice.VertexDeclaration = vpcDecl;
            drawInfo.GraphicsDevice.Vertices[0].SetSource(axisBuffer, 0, VertexPositionColor.SizeInBytes);
            primitiveEffect.Parameters["WorldViewProj"].SetValue((world * drawInfo.Camera.View) * drawInfo.Camera.Projection);
            primitiveEffect.Parameters["Color"].SetValue(Color.White.ToVector4());
            primitiveEffect.Begin();

            foreach (EffectPass pass in primitiveEffect.CurrentTechnique.Passes)
            {
                pass.Begin();
                drawInfo.GraphicsDevice.DrawPrimitives(PrimitiveType.LineList, 0, axisLineCount);
                pass.End();
            }

            primitiveEffect.End();
        }


        public void DrawOrientedBoundingBox(OrientedBoundingBox obb, DrawInfo drawInfo, Color tintColor)
        {
            DrawOrientedBoundingBox(obb.Rotation, obb.Extent, obb.Center, drawInfo, tintColor);
        }

        public void DrawOrientedBoundingBox(Matrix rotation, Vector3 extent, Vector3 position, DrawInfo drawInfo, Color tintColor)
        {
            Matrix world = Matrix.CreateScale(extent) * rotation * Matrix.CreateTranslation(position);

            drawInfo.GraphicsDevice.VertexDeclaration = vpcDecl;
            drawInfo.GraphicsDevice.Vertices[0].SetSource(obbBuffer, 0, VertexPositionColor.SizeInBytes);
            primitiveEffect.Parameters["WorldViewProj"].SetValue((world * drawInfo.Camera.View) * drawInfo.Camera.Projection);
            primitiveEffect.Parameters["Color"].SetValue(tintColor.ToVector4());
            primitiveEffect.Begin();

            foreach (EffectPass pass in primitiveEffect.CurrentTechnique.Passes)
            {
                pass.Begin();
                drawInfo.GraphicsDevice.DrawPrimitives(PrimitiveType.LineList, 0, obbLineCount);
                pass.End();
            }

            primitiveEffect.End();

            DrawAxisGismo(rotation, extent, position, drawInfo);
        }

        public void DrawNodeBoundingBoxes(ObjectNode node, DrawInfo drawInfo)
        {
            DrawNodeBoundingBoxes(node, drawInfo, Color.White);
        }

        public void DrawNodeBoundingBoxes(ObjectNode node, DrawInfo drawInfo, Color color)
        {
            DrawOrientedBoundingBox(node.OrientedBoundingBox, drawInfo, color);

            foreach (SceneNode childNode in node.ChildNodes)
            {
                ObjectNode modelChildNode = childNode as ObjectNode;
                if (modelChildNode != null)
                {
                    DrawNodeBoundingBoxes(modelChildNode, drawInfo, Color.LightGreen);
                }
            }
        }

    }
}
