using System;
using System.Collections.Generic;
using System.Text;
using GameEngine.Data;
using GameEngine.Level;
using System.ComponentModel;
using Microsoft.Xna.Framework;
using System.Windows.Forms;
using Editor.Data;
using GameEngine.Objects;

namespace Editor
{
    public class PhysicsPolygonNode
    {
        private PhysicsPolygon polygon;

        [Browsable(false)]
        public PhysicsPolygon Polygon
        {
            get { return polygon; }
            set { polygon = value; }
        }

        public string Name
        {
            get { return polygon.Name; }
            set { polygon.Name = value; }
        }

        public PhysicsPolygonNode(PhysicsPolygon polygon)
        {
            this.polygon = polygon;
        }

        public PhysicsPolygonNode(PhysicsPolygon polygon, string name)
        {
            this.polygon = polygon;
            this.Name = name;
        }

        public PhysicsPolygonData CreateData()
        {
            PhysicsPolygonData propData = new PhysicsPolygonData(Name, polygon.Points);
            return propData;
        }
    }


    public class StaticPropNode : GameObjectNode
    {
        private string file;
        [ReadOnly(true)]
        public string File
        {
            get { return file; }
            set { file = value; }
        }
        public StaticPropNode(StaticProp prop, string file)
            : base(prop)
        {
            this.file = file;
        }

        public override GameObjectData CreateData()
        {
            GameObjectData propData = new StaticPropData(file, Name, Position, Scale, obj.Rotation);
            return propData;
        }
    }

    public abstract class GameObjectNode
    {
        protected GameObject obj;


        public string Name
        {
            get { return obj.Name; }
            set { obj.Name = value; }
        }

        public Vector3 Position
        {
            get { return obj.Position; }
            set { obj.Position = value; }
        }

        public Vector3 Scale
        {
            get { return obj.Scale; }
            set { obj.Scale = value; }
        }

        [Browsable(false)]
        public Quaternion Rotation
        {
            get { return obj.Rotation; }
        }

        [Browsable(false)]
        public GameObject GameObject { get { return obj; } }

        public GameObjectNode(GameObject obj)
        {
            this.obj = obj;
        }


        public abstract GameObjectData CreateData();
    }


    public class Prop
    {
        private string name = "";

        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        private string value = "";

        public string Value
        {
            get { return value; }
            set { this.value = value; }
        }
        public Prop()
        {

        }
        public override string ToString()
        {
            return name;
        }
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }



    public class PhysicsPropNode : GameObjectNode
    {
        public List<Prop> sager = new List<Prop>();
        public List<Prop> Sager
        {
            get { return sager; }
            set { sager = value; }
        }


        public float Mass
        {
            get { return ((PhysicsPropStub)obj).Mass; }
            set { ((PhysicsPropStub)obj).Mass = value; }
        }

        private string file;

        [ReadOnly(true)]
        public string File
        {
            get { return file; }
            set { file = value; }
        }


        public PhysicsPropNode(PhysicsPropStub obj, string file) : base(obj)
        {
            this.file = file;
        }
        public override GameObjectData CreateData()
        {
            PhysicsPropStub stub = obj as PhysicsPropStub;
            PhysicsPropData newData = stub.Data;



            newData.Position = Position;
            newData.Scale = Scale;
            newData.Mass = Mass;
            newData.ID = Name;

            return newData;
        }
    }

    public class SceneNode
    {
        private string scriptName;

        public string ScriptName
        {
            get { return scriptName; }
            set { scriptName = value; }
        }

    }

}
