using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using GameEngine.Level;
using Microsoft.Xna.Framework;
using GameEngine;
using Editor.Editor;
using GameEngine.Cameras;

namespace Editor.Tools
{
    public class ToolHandler
    {
        public Tool[] tools;

        private Tool activeTool;

        private SpriteFont screenFont;

        private SelectionHandler selection = new SelectionHandler();

        private SelectionTool selectionTool;

        public SelectionHandler Selection
        {
            get { return selection; }
            set { selection = value; }
        }

        public ToolHandler(OrthoOrbitCamera camera)
        {
            tools = new Tool[6];

            int loadIndex = 0;

            selectionTool = new SelectionTool(camera, this);

            tools[loadIndex++] = selectionTool;

            tools[loadIndex++] = new PolygonTool(camera, this);
            tools[loadIndex++] = new ScaleTool(camera, this);
            tools[loadIndex++] = new RotateTool(camera, this);
            tools[loadIndex++] = new MoveTool(camera, this);
            tools[loadIndex++] = new PolyModifierTool(camera, this);

            activeTool = tools[0];
        }

        public Tool ActiveTool
        {
            get { return activeTool; }
        }

        public T GetTool<T>() where T : Tool
        {
            foreach (Tool t in tools)
            {
                if (t is T)
                    return (T)t;
            }
            return null;
        }

        public void LoadContent(ContentManager content)
        {
            for (int i = 0; i < tools.Length; i++)
            {
                tools[i].LoadContent(content);
            }

            screenFont = content.Load<SpriteFont>("Fonts/Arial");
        }

        public void Draw2D(Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch, DrawInfo drawInfo)
        {
            string toolMsg = "Tool: " + activeTool.Name;

            spriteBatch.DrawString(screenFont, toolMsg, new Vector2(drawInfo.GraphicsDevice.Viewport.Width - screenFont.MeasureString(toolMsg).X - 5, 5), Color.White);

            activeTool.Draw2D(spriteBatch, drawInfo);
        }

        public void SelectTool(string name)
        {
            for (int i = 0; i < tools.Length; i++)
            {
                if (tools[i].Name == name)
                {
                    SelectTool(i);
                }
            }
        }

        public void SelectTool(int i)
        {
            if (i >= 0 && i < tools.Length)
            {
                activeTool.ToolEnd();
                activeTool = tools[i];
                activeTool.ToolStart();

            }
        }

        // Method for swithcing temporarily to the SelectionTool
        // Meant to be called from the HandleInput methods from other tools
        public void SwitchTemporarilyToSelection(GameTime gameTime)
        {
            selectionTool.ReturnTool = activeTool.Name;
            SelectTool("SelectionTool");
            activeTool.HandleInput(gameTime);
        }
    }
}
