﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Editor.Editor;
using Microsoft.Build.BuildEngine;
using System.IO;

namespace Editor
{
    public class ContentItemWrapper : IComparable
    {
        BuildItem item;
        ContentProjectWrapper project;
        public ContentProjectWrapper Project { get { return project; } }
        public BuildItem WrappedItem { get { return item; } }
        public string Name
        {
            get { return item.GetMetadata("Name"); }
            set { item.SetMetadata("Name", value); }
        }

        public string AssetPath
        {
            get { return Path.Combine(Path.GetDirectoryName(Include), Name); }
        }

        public string Include
        {
            get { return item.Include; }
            set { item.Include = value; }
        }

        public string FullPath { get { return Path.Combine(Path.GetDirectoryName(project.File), Include); } }

        public string Importer
        {
            get { return item.GetMetadata("Importer"); }
            set { item.SetMetadata("Importer", value); }
        }

        public string Processor
        {
            get { return item.GetMetadata("Processor"); }
            set { item.SetMetadata("Processor", value); }
        }

        public ContentItemWrapper(BuildItem item, ContentProjectWrapper project)
        {
            this.project = project;
            this.item = item;
        }

        #region IComparable Members

        public int CompareTo(object obj)
        {
            ContentItemWrapper that = obj as ContentItemWrapper;
            if (that == null)
                return 0;

            int i = FileUtil.SplitPath(this.Include).Length.CompareTo(FileUtil.SplitPath(that.Include).Length);

            if (i != 0)
                return -i;

            return this.Include.CompareTo(that.Include);
        }



        #endregion
 
    }

}
