using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Build.BuildEngine;
using System.IO;
using System.Runtime.InteropServices;

namespace Editor
{


    public class ContentProjectWrapper
    {
        string contentProjectFile;
        public string File { get { return contentProjectFile; } }
        public string Directory { get { return Path.GetDirectoryName(contentProjectFile); } }
        Project project;
        Engine engine;
        public Engine Engine { get { return engine; } }
        private List<ContentItemWrapper> items = new List<ContentItemWrapper>();

        public List<ContentItemWrapper> Items
        {
            get { return items; }
        }

        public string OutputPath { get { return project.GetEvaluatedProperty("OutputPath"); } }


        public ContentProjectWrapper(string contentProjectFile)
        {
            this.contentProjectFile = contentProjectFile ;
            ReloadProject();
        }

        ~ContentProjectWrapper()
        {
            if (engine != null)
            {
                engine.UnloadAllProjects();
                engine = null;
            }

        }

        public void ReloadProject()
        {
            LoadProject(contentProjectFile);
        }

        void LoadProject(string file)
        {
            if (engine != null)
            {
                engine.UnloadAllProjects();
                engine = null;
            }
            engine = new Engine();
            engine.DefaultToolsVersion = "3.5";
            project = new Project(engine);
            project.Load(file);
            items.Clear();

            foreach (BuildItemGroup group in project.ItemGroups)
            {
                foreach (BuildItem item in group)
                {
                    if (item.Name == "Compile")
                    {
                        ContentItemWrapper itemWrapper = new ContentItemWrapper(item, this);
                        items.Add(itemWrapper);
                    }
                }
            }
            items.Sort();
        }

        public ContentItemWrapper AddItem(string itemName, string itemPath, string importer, string processor)
        {
            BuildItem item = project.AddNewItem("Compile", itemPath);
            item.SetMetadata("Name", itemName);
            ContentItemWrapper wrapper = new ContentItemWrapper(item, this);
            wrapper.Processor = processor;
            wrapper.Importer = importer;
            items.Add(wrapper);
            return wrapper;
        }

        public void RemoveItem(ContentItemWrapper item)
        {
            project.RemoveItem(item.WrappedItem);

            bool i = items.Contains(item);

            items.Remove(item);
        }
        

        public void Save()
        {
            project.Save(contentProjectFile);
        }
        /*
        public void BuildProject(string outputPath)
        {
            project.SetProperty("OutputPath", outputPath);
            project.Build();
        }

        public void BuildProject()
        {
            BuildProject("");
        }*/

    }
}