using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Build.BuildEngine;
using System.IO;
using System.Runtime.InteropServices;
using GameEngine;

namespace Editor.Editor
{
    public class XNAProjectWrapper : IDisposable
    {
        string projectFile;
        Project project;
        Engine engine;
        public ContentProjectWrapper Content { get; private set; }
        public Project Project { get { return project; } }
        public Engine Engine { get { return engine; } }
        public string OutputPath { get { return project.GetEvaluatedProperty("OutputPath"); } }
        public string TargetFileName { get { return project.EvaluatedProperties["TargetFileName"].FinalValue; } }
        public string ProjectFile { get { return projectFile; } }

        public XNAProjectWrapper(string file)
        {
            this.projectFile = file;
            Reload();
        }

        public void Reload()
        {
            Dispose();

            engine = new Engine();
            
            project = new Project(engine);
            project.Load(projectFile);
            foreach (BuildItemGroup group in project.ItemGroups)
            {
                foreach (BuildItem item in group)
                {
                    if (item.Name == "NestedContentProject")
                    {
                        Content = new ContentProjectWrapper(Path.Combine(Path.GetDirectoryName(projectFile), item.Include));
                    }
                }
            }
        }

        ~XNAProjectWrapper()
        {
            Dispose();
        }

        public void Dispose()
        {
            if (engine != null)
            {
                engine.UnloadAllProjects();
                engine.Shutdown();
                engine = null;
            }
        }
    }
}
