﻿using System;
using System.Collections.Generic;
using System.Text;
using GameEngine.Animation;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework;

namespace GameContentPipeline
{
    public class AnimationDataReader : ContentTypeReader<AnimationData>
    {
        protected override AnimationData Read(ContentReader input, AnimationData existingInstance)
        {
            AnimationData data = new AnimationData();

            data.Type = input.ReadObject<AnimationType>();
            data.AminationTracks = input.ReadObject<Dictionary<string, AnimationTrack>>();

            if (data.Type == AnimationType.Skinnned)
            {
                data.BindPoses = input.ReadObject<List<Matrix>>();
                data.InverseBindPoses = input.ReadObject<List<Matrix>>();
                data.SkeletonHierarchy = input.ReadObject<List<int>>();            
            }
            
            return data;
        }
    }

    public class AnimationTrackReader : ContentTypeReader<AnimationTrack>
    {
        protected override AnimationTrack Read(ContentReader input, AnimationTrack existingInstance)
        {
            TimeSpan duration = input.ReadObject<TimeSpan>();
            List<Keyframe> keyframes = input.ReadObject<List<Keyframe>>();

            return new AnimationTrack(duration, keyframes);
        }
    }

    public class KeyframeReader : ContentTypeReader<Keyframe>
    {
        protected override Keyframe Read(ContentReader input, Keyframe existingInstance)
        {
            int bone = input.ReadObject<int>();
            TimeSpan time = input.ReadObject<TimeSpan>();
            Matrix transform = input.ReadObject<Matrix>();

            return new Keyframe(bone, time, transform);
        }        
    }
}
