﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace GameEngine.Animation
{
    public class StaticModelManipulator
    {
        public Model Model;

        Matrix[] worldTransforms;
        Matrix[] currentTransforms;
        bool[] Dirty;

        public Matrix this[int index]
        {
            get 
            {
                return currentTransforms[index];
            }
            set 
            {
                currentTransforms[index] = value;
                DirtyRecursively(index);
            }
        }

        public Matrix this[string name]
        {
            get
            {
                int i = Model.Bones[name].Index;
                return currentTransforms[i];
            }
            set
            {
                int i = Model.Bones[name].Index;
                currentTransforms[i] = value;
                DirtyRecursively(i);
            }
        }

        private void DirtyRecursively(int index)
        {
            Dirty[index] = true;
            for (int i = 0; i < Model.Bones[index].Children.Count; i++)
            {
                DirtyRecursively(Model.Bones[index].Children[i].Index);
            }
        }

        public StaticModelManipulator(Model model)
        {
            this.Model = model;
            
            Dirty = new bool[model.Bones.Count];
            worldTransforms = new Matrix[model.Bones.Count];
            currentTransforms = new Matrix[model.Bones.Count];

            model.CopyAbsoluteBoneTransformsTo(worldTransforms);
            model.CopyBoneTransformsTo(currentTransforms);
        }

        public void ResolveDirty()
        {
            for (int i = 0; i < worldTransforms.Length; i++)
            {
                if (!Dirty[i])
                    continue;

                if (Model.Bones[i].Parent == null)
                {
                    worldTransforms[i] = currentTransforms[i];
                }
                else
                {
                    worldTransforms[i] = currentTransforms[i] * worldTransforms[Model.Bones[i].Parent.Index];
                }
                Dirty[i] = false;
            }
        }

        public Matrix GetWorldTransformation(int index)
        {
            if (Dirty[index])
                ResolveDirty();

            return worldTransforms[index];
        }


    }
}
