﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;

namespace GameEngine.Helpers
{
    public abstract class Geometry : IDisposable
    {
        public VertexBuffer buffer;
        public IndexBuffer indices;
        public VertexDeclaration decl;
        public PrimitiveType primitiveType;
        public int primitiveCount, vertexCount;

        public void Draw(GraphicsDevice device)
        {
            device.DrawIndexedPrimitives(primitiveType, 0, 0, vertexCount, 0, primitiveCount);
        }

        public void Dispose()
        {
            Util.NullDispose(ref buffer);
            Util.NullDispose(ref indices);
            Util.NullDispose(ref decl);
        }

        public void SetSource(GraphicsDevice device)
        {
            device.VertexDeclaration = decl;
            device.Vertices[0].SetSource(buffer, 0, VertexPositionNormalTexture.SizeInBytes);
            device.Indices = indices;
        }

        public void Draw(GraphicsDevice device, Effect effect)
        {
            SetSource(device);
            effect.Begin();

            for (int i = 0; i < effect.CurrentTechnique.Passes.Count; i++)
            {
                effect.CurrentTechnique.Passes[i].Begin();

                Draw(device);

                effect.CurrentTechnique.Passes[i].End();
            }

            effect.End();
        }


    }
}
