﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine.Xml;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework;

namespace GameEngine.Helpers
{
    public class PolygonDrawer
    {
        VertexDeclaration vertDecl;
        Effect primitivesEffect;
        
        public PolygonDrawer(Effect primitivesEffect, GraphicsDevice device)
        {
            this.primitivesEffect = primitivesEffect;
            vertDecl = new VertexDeclaration(device, VertexPositionColor.VertexElements);
        }

        public void DrawPolys(ForestHandler forest, DrawInfo di)
        {
            DrawPolys(forest, di, new PolygonNode[0]);
        }
        public void DrawPolys(ForestHandler forest, DrawInfo di, IEnumerable<PolygonNode> selection)
        {
            //di.GraphicsDevice.RenderState.DepthBufferEnable = false;
            foreach (var node in forest.NodesDepthFirst)
            {
                PolygonNode poly = node as PolygonNode;
                if (poly == null)
                    continue;

                DrawSinglePoly(poly, di, selection.Contains(poly) ? Color.Red : Color.Blue);
            }
            //di.GraphicsDevice.RenderState.DepthBufferEnable = true;
        }


        //public void DrawGeoms(PhysicsSimulator simulator, DrawInfo di)
        //{
        //    di.GraphicsDevice.RenderState.DepthBufferEnable = false;

        //    foreach (var g in simulator.GeomList)
        //    {
        //        VertexPositionColor[] vertices = new VertexPositionColor[g.WorldVertices.Count + 1];
        //        for (int i = 0; i < g.WorldVertices.Count; i++)
        //        {
        //            vertices[i].Color = Color.Red;
        //            vertices[i].Position = new Vector3(g.WorldVertices[i], 0);
        //        }


        //        vertices[g.WorldVertices.Count] = vertices[0];

        //        di.GraphicsDevice.VertexDeclaration = vertDecl;
        //        primitivesEffect.Parameters["Color"].SetValue(Vector4.One);
        //        primitivesEffect.Parameters["WorldViewProj"].SetValue(di.View * di.Projection);
        //        primitivesEffect.Begin();

        //        foreach (EffectPass pass in primitivesEffect.CurrentTechnique.Passes)
        //        {
        //            pass.Begin();
        //            di.GraphicsDevice.DrawUserPrimitives<VertexPositionColor>(PrimitiveType.LineStrip, vertices, 0, g.WorldVertices.Count);
        //            pass.End();
        //        }
        //        primitivesEffect.End();
        //    }
        //    di.GraphicsDevice.RenderState.DepthBufferEnable = true;



        //}

        public void DrawSinglePoly(PolygonNode node, DrawInfo drawInfo, Color color)
        {
            VertexPositionColor[] vertices = new VertexPositionColor[node.Points.Count + 1];
            for (int i = 0; i < node.Points.Count; i++)
            {
                vertices[i].Color = color;
                vertices[i].Position = Vector3.Transform(new Vector3(node.Points[i].X, 0, node.Points[i].Y), node.World);
            }
            vertices[node.Points.Count] = vertices[0];

            drawInfo.GraphicsDevice.VertexDeclaration = vertDecl;
            primitivesEffect.Parameters["Color"].SetValue(Vector4.One);
            primitivesEffect.Parameters["WorldViewProj"].SetValue(drawInfo.Camera.View * drawInfo.Camera.Projection);
            primitivesEffect.Begin();

            foreach (EffectPass pass in primitivesEffect.CurrentTechnique.Passes)
            {
                pass.Begin();
                drawInfo.GraphicsDevice.DrawUserPrimitives<VertexPositionColor>(PrimitiveType.LineStrip, vertices, 0, node.Points.Count);
                pass.End();
            }
            primitivesEffect.End();
        }


    }
}
