﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace GameEngine.Helpers
{
    public class PrimitivesDrawer
    {
        VertexDeclaration vertDecl;
        Effect primitivesEffect;


        int circleSections = 128;
        VertexPositionColor[] circleVertices;

        public PrimitivesDrawer(Effect primitivesEffect, GraphicsDevice device)
        {
            this.primitivesEffect = primitivesEffect;
            vertDecl = new VertexDeclaration(device, VertexPositionColor.VertexElements);

            InitializeCircleVertices();
            InitializeRayVertices();
        }

        public void InitializeCircleVertices()
        {
            circleVertices = new VertexPositionColor[circleSections + 1];

            float rotationPerPoint = (float)Math.PI * 2 / circleSections;

            for (int i = 0; i < circleSections; i++)
            {
                Matrix rotation = Matrix.CreateFromAxisAngle(Vector3.UnitY, i*rotationPerPoint);
                circleVertices[i].Position = Vector3.Transform(Vector3.UnitX, rotation);
                circleVertices[i].Color = Color.White;
            }
            circleVertices[circleSections].Position = Vector3.UnitX;
            circleVertices[circleSections].Color = Color.Blue;
        }

        public void DrawCircle(Vector2 center, float radius, Color color, DrawInfo drawInfo)
        {
            Matrix world = Matrix.CreateScale(radius) * Matrix.CreateTranslation(new Vector3(center.X, 0, center.Y));

            drawInfo.GraphicsDevice.VertexDeclaration = vertDecl;
            primitivesEffect.Parameters["Color"].SetValue(color.ToVector4());
            primitivesEffect.Parameters["WorldViewProj"].SetValue(world * drawInfo.Camera.View * drawInfo.Camera.Projection);
            primitivesEffect.Begin();

            foreach (EffectPass pass in primitivesEffect.CurrentTechnique.Passes)
            {
                pass.Begin();
                drawInfo.GraphicsDevice.DrawUserPrimitives<VertexPositionColor>(PrimitiveType.LineStrip, circleVertices, 0, circleSections);
                pass.End();
            }
            primitivesEffect.End();
        }

        public void DrawCircle(BoundingCircle circle, Color color, DrawInfo drawInfo) 
        {
            DrawCircle(circle.Center, circle.Radius, color, drawInfo);
        }

        VertexPositionColor[] rayVertices;

        public void InitializeRayVertices() 
        {
            rayVertices = new VertexPositionColor[6];
            rayVertices[0].Position = Vector3.Zero;
            rayVertices[1].Position = rayVertices[2].Position = rayVertices[4].Position =new Vector3(100, 0, 0);
            rayVertices[3].Position = new Vector3(95, 0, 5);
            rayVertices[5].Position = new Vector3(95, 0, -5);

            for (int i = 0; i < 6; i++)
            {
                rayVertices[i].Color = Color.White;
            }
        }

        public void DrawRay(CollisionRay ray, Color color, DrawInfo drawInfo)
        {

            Matrix rotation = Matrix.CreateFromAxisAngle(Vector3.UnitY, ray.AngleFromXAxis);
            

            Matrix world = rotation * Matrix.CreateTranslation(new Vector3(ray.Position.X, 0, ray.Position.Y));

            drawInfo.GraphicsDevice.VertexDeclaration = vertDecl;
            primitivesEffect.Parameters["Color"].SetValue(color.ToVector4());
            primitivesEffect.Parameters["WorldViewProj"].SetValue(world * drawInfo.Camera.View * drawInfo.Camera.Projection);
            primitivesEffect.Begin();

            foreach (EffectPass pass in primitivesEffect.CurrentTechnique.Passes)
            {
                pass.Begin();
                drawInfo.GraphicsDevice.DrawUserPrimitives<VertexPositionColor>(PrimitiveType.LineList, rayVertices, 0, 3);
                pass.End();
            }
            primitivesEffect.End();
        }

    }
}
