﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace GameEngine.Helpers
{
    public class QuadNormal : Geometry, IDisposable
    {
        public Vector3 Normal { get; private set; }
        public Vector3 Center { get; private set; }
        private QuadNormal()
        {
        }

        ~QuadNormal()
        {
            Dispose();
        }

        public static QuadNormal GenerateWall(GraphicsDevice device, Vector3 start, Vector3 end, Vector3 height)
        {
            return GenerateWall(device, start, end, height, Vector2.Zero, Vector2.One);

        }

        public static QuadNormal GenerateWall(GraphicsDevice device, Vector3 start, Vector3 end, Vector3 height, Vector2 texCoordTopLeft, Vector2 texCoordBottomRight)
        {
            QuadNormal q = new QuadNormal();
            q.buffer = new VertexBuffer(device, typeof(VertexPositionNormalTexture), 4, BufferUsage.WriteOnly);

            Vector3 normal = Vector3.Normalize(height);

            VertexPositionNormalTexture[] vertices = new VertexPositionNormalTexture[4] 
            {
                new VertexPositionNormalTexture(start + height, normal, texCoordTopLeft),
                new VertexPositionNormalTexture(end + height, normal, new Vector2(texCoordBottomRight.X, texCoordTopLeft.Y)),
                new VertexPositionNormalTexture(end, normal, texCoordBottomRight),
                new VertexPositionNormalTexture(start, normal,  new Vector2(texCoordTopLeft.X, texCoordBottomRight.Y))
            };
            q.buffer.SetData(vertices);

            q.indices = new IndexBuffer(device, typeof(short), 4, BufferUsage.WriteOnly);
            q.indices.SetData(new short[] { 0, 1, 3, 2 });
            q.decl = new VertexDeclaration(device, VertexPositionNormalTexture.VertexElements);
            q.Normal = Vector3.Cross(height, end - start);
            q.Normal = Vector3.Normalize(q.Normal);
            q.Center = (end + start) / 2f + height / 2f;
            q.primitiveCount = 2;
            q.vertexCount = 4;

            return q;

        }

        public static QuadNormal GenerateXZ(GraphicsDevice device, float scale, float height, Vector2 textureOffset, Vector2 textureSize, Vector4 color) 
        {
            QuadNormal q = new QuadNormal();
            q.buffer = new VertexBuffer(device, typeof(VertexPositionNormalTexture), 4, BufferUsage.WriteOnly);

            Vector3 normal = Vector3.UnitY;

            VertexPositionNormalTexture[] vertices = new VertexPositionNormalTexture[4] 
            {
                new VertexPositionNormalTexture(new Vector3(-.5f * scale,height,-.5f * scale) , normal, textureSize == Vector2.Zero ? Vector2.Zero : new Vector2(textureOffset.X / textureSize.X, textureOffset.Y / textureSize.Y)),
                new VertexPositionNormalTexture(new Vector3(.5f * scale,height,-.5f * scale), normal, textureSize == Vector2.Zero ? Vector2.UnitX : new Vector2(textureOffset.X / textureSize.X + scale / textureSize.X, textureOffset.Y / textureSize.Y)),
                new VertexPositionNormalTexture(new Vector3(.5f * scale,height,.5f * scale), normal, textureSize == Vector2.Zero ? Vector2.One : new Vector2(textureOffset.X / textureSize.X + scale / textureSize.X, textureOffset.Y / textureSize.Y + scale / textureSize.Y)),
                new VertexPositionNormalTexture(new Vector3(-.5f * scale,height,.5f * scale), normal,  textureSize == Vector2.Zero ? Vector2.UnitY : new Vector2(textureOffset.X / textureSize.X, textureOffset.Y / textureSize.Y + scale / textureSize.Y))
            };
            q.buffer.SetData(vertices);

            q.indices = new IndexBuffer(device, typeof(short), 4, BufferUsage.WriteOnly);
            q.indices.SetData(new short[] { 0, 1, 3, 2 });
            q.decl = new VertexDeclaration(device, VertexPositionNormalTexture.VertexElements);
            q.primitiveCount = 2;
            q.vertexCount = 4;
            q.primitiveType = PrimitiveType.TriangleStrip;
            return q;
        }

        public Matrix GetXZWorld(Vector3 topLeft, float width, float height)
        {
            //  + new Vector3(width / 2f,0 , height / 2f)
            return Matrix.CreateScale(new Vector3(width, 0, height)) * Matrix.CreateTranslation(topLeft);
        }


        
    }
}
