using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using GameEngine.Level;

namespace GameEngine.Helpers
{
    public class SelectionBox
    {
        VertexBuffer buffer;
        VertexDeclaration decl;
        Effect primitiveEffect;
        IndexBuffer indices;

        Vector3 min, max;

        Matrix world;
        public Vector3 Position;
        public Vector3 Scale = Vector3.One;
        public Quaternion Rotation;

        public Color Color = Color.Yellow;
        public SelectionBox(Vector3 min, Vector3 max)
        {
            this.min = Vector3.Min(min, max);
            this.max = Vector3.Max(max, min);
        }

        public void Draw(DrawInfo di)
        {

            di.GraphicsDevice.VertexDeclaration = decl;
            di.GraphicsDevice.Vertices[0].SetSource(buffer, 0, VertexPositionColor.SizeInBytes);
            di.GraphicsDevice.Indices = indices;
            primitiveEffect.Parameters["WorldViewProj"].SetValue((world * di.Camera.View) * di.Camera.Projection);
            primitiveEffect.Parameters["Color"].SetValue(Color.ToVector4());
            primitiveEffect.Begin();

            foreach (EffectPass pass in primitiveEffect.CurrentTechnique.Passes)
            {

                pass.Begin();
                di.GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.LineList, 0, 0, 8, 0, 4);
                di.GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.LineStrip, 0, 0, 4, 8, 4);
                di.GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.LineStrip, 4, 0, 4, 8, 4);
                pass.End();
            }
            primitiveEffect.End();
        }

        public void Initialize(GraphicsDevice graphicsDevice, Effect primitivesEffect)
        {
            Unload();
            this.primitiveEffect = primitivesEffect;
            buffer = new VertexBuffer(graphicsDevice, typeof(VertexPositionColor), 8, BufferUsage.WriteOnly);
            indices = new IndexBuffer(graphicsDevice, typeof(short), 13, BufferUsage.WriteOnly);
            
            VertexPositionColor[] vertices = new VertexPositionColor[8];

            

            vertices[0] = new VertexPositionColor(min, Color.White);
            vertices[1] = new VertexPositionColor(new Vector3(max.X, min.Y, min.Z), Color.White);
            vertices[2] = new VertexPositionColor(new Vector3(max.X, min.Y, max.Z), Color.White);
            vertices[3] = new VertexPositionColor(new Vector3(min.X, min.Y, max.Z), Color.White);
            vertices[4] = new VertexPositionColor(new Vector3(min.X, max.Y, min.Z), Color.White);
            vertices[5] = new VertexPositionColor(new Vector3(max.X, max.Y, min.Z), Color.White);
            vertices[6] = new VertexPositionColor(max, Color.White);
            vertices[7] = new VertexPositionColor(new Vector3(min.X, max.Y, max.Z), Color.White);

            short[] indix = new short[] { 0, 4, 1, 5, 2, 6, 3, 7,    0, 1, 2, 3, 0, };

            indices.SetData(indix);
            buffer.SetData(vertices);

            decl = new VertexDeclaration(graphicsDevice, VertexPositionColor.VertexElements);
        }

        public void Update()
        {
            world = Matrix.CreateFromQuaternion(Rotation) * Matrix.CreateScale(Scale) * Matrix.CreateTranslation(Position);//  ;
        }

        public void Unload()
        {
            if (buffer != null)
            {
                buffer.Dispose();
                buffer = null;
            }
            if (decl != null)
            {
                decl.Dispose();
                decl = null;
            }
            if (indices != null)
            {
                indices.Dispose();
                indices = null;
            }

        }
    }
}
