﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using GamePrototype;

namespace GameEngine.Helpers
{
    public class TesselatedPolygon : IDisposable
 {
        VertexBuffer buffer;
        IndexBuffer indices;
        VertexDeclaration decl;
        int triangles;
        int pointCount;


        private TesselatedPolygon()
        {
        }

        ~TesselatedPolygon()
        {
            Dispose();
        }

        private static Vector4 GenerateSize(List<Vector2> points)
        {
            float minx = float.MaxValue,maxx = float.MinValue;
            float miny = float.MaxValue,maxy = float.MinValue;

            foreach (var item in points)
            {
                minx = MathHelper.Min(minx, item.X);
                maxx = MathHelper.Max(maxx, item.X);
                miny = MathHelper.Min(miny, item.Y);
                maxy = MathHelper.Max(maxy, item.Y);
            }
            return new Vector4(minx, miny, maxx - minx, maxy - miny);
        }

        public static TesselatedPolygon Generate(GraphicsDevice device, List<Vector2> points, Vector3 offset, Vector2 texOffset, Vector2 texSize, Vector4 color)
        {
            Vector4 box = GenerateSize(points);
            Vector2 topLeft = new Vector2(box.X, box.Y);
            if(texSize == Vector2.Zero)
            {
                texSize = new Vector2(box.Z, box.W);
                texOffset = Vector2.Zero;
            }

            Color c = new Color(color);
            TesselatedPolygon p = new TesselatedPolygon();

            List<int> ix = new List<int>();
            Triangulator2Dflipcode.Process(points, ix);
            

            p.indices = new IndexBuffer(device, typeof(int), ix.Count, BufferUsage.WriteOnly);
            p.indices.SetData(ix.ToArray());

            p.buffer = new VertexBuffer(device, typeof(VertexPositionColorTexture), points.Count, BufferUsage.WriteOnly);
            VertexPositionColorTexture[] v = new VertexPositionColorTexture[points.Count];

            for (int i = 0; i < points.Count; i++)
            {
                v[i] = new VertexPositionColorTexture(new Vector3(points[i].X, 0, points[i].Y) + offset, c, (points[i] - topLeft + texOffset) / texSize);
            }
            p.buffer.SetData(v);

            p.decl = new VertexDeclaration(device, VertexPositionColorTexture.VertexElements);
            p.pointCount = points.Count;
            p.triangles = ix.Count / 3;
            return p;
        }

        public void SetSource(GraphicsDevice device)
        {
            device.VertexDeclaration = decl;
            device.Vertices[0].SetSource(buffer, 0, VertexPositionColorTexture.SizeInBytes);
            device.Indices = indices;
        }

        public void Draw(GraphicsDevice device, Effect effect)
        {
            SetSource(device);
            effect.Begin();

            for (int i = 0; i < effect.CurrentTechnique.Passes.Count; i++)
            {
                effect.CurrentTechnique.Passes[i].Begin();

                Draw(device);

                effect.CurrentTechnique.Passes[i].End();
            }

            effect.End();
        }

        public void Draw(GraphicsDevice device)
        {
            device.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, pointCount, 0, triangles);
        }

        public void Dispose()
        {
            Util.NullDispose(ref buffer);
            Util.NullDispose(ref indices);
            Util.NullDispose(ref decl);
        }
        
    }
}
