﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using GamePrototype;

namespace GameEngine.Helpers
{
    public class TesselatedPolygonNormals : Geometry, IDisposable
    {
        private TesselatedPolygonNormals()
        {
        }

        ~TesselatedPolygonNormals()
        {
            Dispose();
        }

        private static Vector4 GenerateSize(List<Vector2> points)
        {
            float minx = float.MaxValue,maxx = float.MinValue;
            float miny = float.MaxValue,maxy = float.MinValue;

            foreach (var item in points)
            {
                minx = MathHelper.Min(minx, item.X);
                maxx = MathHelper.Max(maxx, item.X);
                miny = MathHelper.Min(miny, item.Y);
                maxy = MathHelper.Max(maxy, item.Y);
            }
            return new Vector4(minx, miny, maxx - minx, maxy - miny);
        }

        public static TesselatedPolygonNormals Generate(GraphicsDevice device, List<Vector2> points, Vector3 offset, Vector2 texOffset, Vector2 texSize, Vector4 color)
        {
            Vector4 box = GenerateSize(points);
            Vector2 topLeft = new Vector2(box.X, box.Y);
            if(texSize == Vector2.Zero)
            {
                texSize = new Vector2(box.Z, box.W);
                texOffset = Vector2.Zero;
            }

            Color c = new Color(color);
            TesselatedPolygonNormals p = new TesselatedPolygonNormals();

            List<int> ix = new List<int>();
            Triangulator2Dflipcode.Process(points, ix);
            

            p.indices = new IndexBuffer(device, typeof(int), ix.Count, BufferUsage.WriteOnly);
            p.indices.SetData(ix.ToArray());

            p.buffer = new VertexBuffer(device, typeof(VertexPositionNormalTexture), points.Count, BufferUsage.WriteOnly);
            VertexPositionNormalTexture[] v = new VertexPositionNormalTexture[points.Count];

            for (int i = 0; i < points.Count; i++)
            {
                v[i] = new VertexPositionNormalTexture(new Vector3(points[i].X, 0, points[i].Y) + offset, Vector3.UnitY, (points[i] - topLeft + texOffset) / texSize);
            }
            p.buffer.SetData(v);

            p.decl = new VertexDeclaration(device, VertexPositionNormalTexture.VertexElements);
            p.vertexCount = points.Count;
            p.primitiveCount = ix.Count / 3;
            p.primitiveType = PrimitiveType.TriangleList;
            return p;
        }


     
        
    }
}
