﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine.Xml;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework;
using GameEngine.LevelGeneration;

namespace GameEngine.Helpers
{
    public class TilesetDebug
    {
        Effect primitivesEffect;
        Quad marker;

        public TilesetDebug(Effect primitivesEffect, GraphicsDevice device)
        {
            this.primitivesEffect = primitivesEffect;
            marker = Quad.GenerateXZ(device, 1, 0, Vector2.Zero, Vector2.Zero, new Vector4(1,0,0,1));
        }

        public void DrawMarkers(float tileSize, float exitSize, TileType tt, DrawInfo di)
        {
            const float markerWidth = 10;

            Vector3 horizontal = new Vector3(tileSize / 2f + markerWidth / 2f, 0, 0);
            Vector3 vertical = new Vector3(0, 0, + tileSize / 2f + markerWidth / 2f);

            primitivesEffect.Parameters["Color"].SetValue(Vector4.One);
            
            // left
            DrawMarker(-horizontal, markerWidth, exitSize, di);
            
            // top
            if (((uint)tt & 2) == 2)
                DrawMarker(-vertical, exitSize, markerWidth, di);

            // right
            if (((uint)tt & 4) == 4)
                DrawMarker(horizontal, markerWidth, exitSize, di);

            // bottom
            if (((uint)tt & 8) == 8)
                DrawMarker(vertical, exitSize, markerWidth, di);
        }

        private void DrawMarker(Vector3 pos, float width, float height, DrawInfo di)
        {
            primitivesEffect.Parameters["WorldViewProj"].SetValue(marker.GetXZWorld(pos, width,height) * di.Camera.View * di.Camera.Projection);
            marker.Draw(di.GraphicsDevice, primitivesEffect);
        }


    }
}
