﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using GamePrototype.GameLogic;

namespace GameEngine.Physics
{
    public class ParticleRenderer
    {
        public const int MAX_PARTICLES = 1000;
        private VertexPositionColor[] pointList;
        private Effect effect;
        private VertexDeclaration decl;
        private List<MuzzleFlashLight> light = new List<MuzzleFlashLight>();
        private int lightCounter = 0;
        
        public ParticleRenderer()
        {
            pointList = new VertexPositionColor[MAX_PARTICLES];
            for (int i = 0; i < MAX_PARTICLES; i++)
            {
                light.Add(new MuzzleFlashLight() { Color = Color.Red, LightIntensity = 0.5f, LightRadius = 100 });
            }
        }

        public void Load(ContentManager content, GraphicsDevice device)
        {
            effect = content.Load<Effect>(EngineFX.RenderGBuffer);
            decl = new VertexDeclaration(device, VertexPositionColor.VertexElements);
        }

        public void DrawAsPoints(DrawInfo di, ParticleSystem ps, Color color, float minSize, float maxSize, float intensity)
        {
            int vertices = SetupPoints(ps, color, di, intensity);
            if (vertices > 0)
            {
                di.GraphicsDevice.VertexDeclaration = decl;
                di.GraphicsDevice.RenderState.PointSize = (float)Util.Random.NextDouble() * (maxSize - minSize) + minSize;
                effect.CurrentTechnique = effect.Techniques["Particles"];
                effect.Parameters["World"].SetValue(Matrix.Identity);

                effect.Begin();
                effect.CurrentTechnique.Passes[0].Begin();

                di.GraphicsDevice.DrawUserPrimitives(PrimitiveType.PointList, pointList, 0, ps.ActiveParticles.Count);

                effect.CurrentTechnique.Passes[0].End();
                effect.End();
            }
        }


        public void DrawAsLines(DrawInfo di, ParticleSystem ps, Color color, float lenght, float intensity)
        {
            int vertices = SetupLines(ps, lenght, color, di, intensity);
            if (vertices > 0)
            {
                di.GraphicsDevice.VertexDeclaration = decl;
                effect.CurrentTechnique = effect.Techniques["Particles"];
                effect.Parameters["World"].SetValue(Matrix.Identity);

                effect.Begin();
                effect.CurrentTechnique.Passes[0].Begin();

                di.GraphicsDevice.DrawUserPrimitives(PrimitiveType.LineList, pointList, 0, ps.ActiveParticles.Count);

                effect.CurrentTechnique.Passes[0].End();
                effect.End();
            }
        }

        public void AddLight(Vector3 position, DrawInfo di, Color color, float intensity)
        {
            light[lightCounter].LightPosition = position;
            light[lightCounter].Color = color;
            light[lightCounter].LightIntensity = intensity;
            di.PointLights.Add(light[lightCounter]);
            lightCounter++;
            if (lightCounter >= MAX_PARTICLES)
                lightCounter = 0;
            
        }

        private int SetupPoints(ParticleSystem system, Color color, DrawInfo di, float intensity)
        {
            int max = Math.Min(system.ActiveParticles.Count, MAX_PARTICLES);
            int c = 0;
            for (int i = 0; i < max; i++)
            {
                pointList[c++] = new VertexPositionColor(system.ActiveParticles[i].Position, color);

                if (intensity > 0)
                    AddLight(system.ActiveParticles[i].Position, di, color, intensity);
            }
            return c;
        }


        private int SetupLines(ParticleSystem system, float length, Color color, DrawInfo di, float intensity)
        {
            int max = Math.Min(system.ActiveParticles.Count, MAX_PARTICLES);
            int c = 0;
            for (int i = 0; i < max; i++)
            {
                pointList[c++] = new VertexPositionColor(system.ActiveParticles[i].Position, color);
                pointList[c++] = new VertexPositionColor(system.ActiveParticles[i].Position - Vector3.Normalize(system.ActiveParticles[i].Direction) * length, color);

                if(intensity > 0)
                    AddLight(system.ActiveParticles[i].Position, di, color, intensity);
            }
            return c;
        }

    }
}
