﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace GameEngine.Physics
{
    public abstract class ParticleSystem
    {
        private float deltaTime;
        public List<ParticleSystemParticle> Particles;
        public List<ParticleSystemParticle> ActiveParticles;
        protected List<ParticleSystemParticle> justFree; 
                
        public ParticleSystem(int maxParticles)
        {
            Particles = new List<ParticleSystemParticle>(maxParticles);
            ActiveParticles = new List<ParticleSystemParticle>(maxParticles);
            justFree = new List<ParticleSystemParticle>();

            for (int i = 0; i < maxParticles; i++)
            {
                Particles.Add(new ParticleSystemParticle());
            }
        }

        public ParticleSystemParticle GetFreeParticle()
        {
            if (Particles.Count < 1)
            {
                return null;
            }
            else
            {
                ParticleSystemParticle p = Particles[0];
                Particles.RemoveAt(0);
                ActiveParticles.Add(p);
                p.Active = true;
                p.LifeTime = 0;
                p.Position = Vector3.Zero;
                p.Direction = Vector3.Zero;
                p.Speed = 0f;

                return p;
            }
        }

        protected void Kill(ParticleSystemParticle p)
        {
            p.Active = false;
            justFree.Add(p);
        }

        protected virtual void OnParticleUpdate(ParticleSystemParticle p, GameTime time) 
        {
            p.LifeTime += deltaTime;
            p.Position += p.Direction * p.Speed * deltaTime;


            if (p.LifeTime > p.MaxLifeTime || p.Active == false)
            {
                Kill(p); 
            }
        }

        public virtual void Update(GameTime gameTime)
        {
            deltaTime = gameTime.ElapsedMillis();

            for (int i = 0; i < ActiveParticles.Count; i++) // Omskriv til at køre baglæns
            {
                OnParticleUpdate(ActiveParticles[i], gameTime);
            }

            if (justFree.Count > 0)
            {
                for (int i = 0; i < justFree.Count; i++)
                {
                    Particles.Add(justFree[i]);
                    ActiveParticles.Remove(justFree[i]);
                }

                justFree.Clear();
            }
        }
    }
}
