using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace GameEngine.ScreenSystem
{
    public class ScreenController
    {
        private Dictionary<string, GameScreen> allScreens;
        private List<GameScreen> activeScreens;
        private List<GameScreen> inactiveScreens;
        
        private string justActivated = "";
        
        public ScreenController()
        {
            allScreens = new Dictionary<string, GameScreen>();
            activeScreens = new List<GameScreen>();
            inactiveScreens = new List<GameScreen>();
        }

        public void LoadScreen(string id, GraphicsDeviceManager graphics)
        {
            GetScreen(id).LoadContent(graphics);
        }

        public void LoadAllScreens(GraphicsDeviceManager graphics)
        {
            foreach (GameScreen gs in allScreens.Values)
            {
                gs.LoadContent(graphics);
            }
        }

        public void UnloadScreen(string id)
        {
            GetScreen(id).UnloadContent();
        }

        public void UnloadAllScreens()
        {
            foreach (GameScreen gs in allScreens.Values)
            {
                gs.UnloadContent();
            }
        }

        public void Draw(GameTime gameTime, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            // Draw the screens. Active screens must be in front, so they are drawn last
            for (int i = 0; i < inactiveScreens.Count; i++)
                inactiveScreens[i].Draw(gameTime, graphics, spriteBatch, spriteFont);
       
            for (int i = 0; i < activeScreens.Count; i++)
                activeScreens[i].Draw(gameTime, graphics, spriteBatch, spriteFont);

            if (inactiveScreens.Count == 0 && activeScreens.Count == 0)
            {
                throw new Exception("Remember to set a screen as active");
            }
        }
        
        public void Update(GameTime gameTime)
        {
            for (int i = 0; i < activeScreens.Count; i++)
            {
                if (activeScreens[i].ID != justActivated)
                {
                    activeScreens[i].Update(gameTime);
                }
            }

            justActivated = "";
        }

        public GameScreen GetScreen(string id)
        {
            return allScreens[id];
        }
                
        public void SetActiveScreen(string id)
        {
            GameScreen s = allScreens[id];
            activeScreens.Add(s);
            inactiveScreens.Remove(s);
            justActivated = id;
        }

        public void SetInactiveScreen(string id) // Still drawn, but not active
        {
            GameScreen s = allScreens[id];
            activeScreens.Remove(s);
            inactiveScreens.Add(s);
        }

        public void SetHiddenScreen(string id)
        {
            GameScreen s = allScreens[id];
            
            activeScreens.Remove(s);
            inactiveScreens.Remove(s);
        }

        public Dictionary<string, GameScreen> Screens { get { return allScreens; } set { allScreens = value; } }	
    }
}
