﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace GameEngine.Xml
{
    public static class NodeReflector
    {
        static List<Type> nodeTypes = null;

        /// <summary>
        /// Gets a list of all node types that can be instanciated using a default constructor
        /// </summary>
        /// <returns>a list of all node types that can be instanciated using a default constructor</returns>
        public static IEnumerable<Type> GetNodeTypes()
        {
            if (nodeTypes == null)
                GenerateNodeTypes();
            return nodeTypes;
        }

        public static IEnumerable<Type> GetNodeTypes(Type subClass)
        {
            if (nodeTypes == null)
                GenerateNodeTypes();

            return nodeTypes.Where(x => x == subClass || x.IsSubclassOf(subClass));
        }

        public static Type GetType(string name)
        {
            foreach (var t in GetNodeTypes())
            {
                if (t.Name == name)
                    return t;
            }
            return null;
        }

        public static bool TryInstantiate(string type, string name, out SceneNode n)
        {
            Type t = GetType(type);
            n = null;
            if (t == null)
                return false;

            n = InstanciateNode(t, name) as SceneNode;
            return n != null;
        }

        public static object InstanciateNode(Type t, string name)
        {
            return t.GetConstructor(new Type[] { typeof(string) }).Invoke(new object[] { name });
        }

        private static void GenerateNodeTypes()
        {
            nodeTypes = new List<Type>();
            foreach (Assembly a in AppDomain.CurrentDomain.GetAssemblies())
            {
                try
                {
                    foreach (Type t in a.GetTypes())
                    {
                        if (t == typeof(SceneNode) || t.IsSubclassOf(typeof(SceneNode)))
                            nodeTypes.Add(t);
                    }
                }
                catch { }
            }
        }

    }
}
