﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using System.ComponentModel;

namespace GameEngine.Xml
{
    public class PointLightNode : ObjectNode, IPointLight, INodeRenderable
    {
        public const float DefaultRadius = 100f;
        public const float DefaultIntensity = 1f;

        public PointLightNode(string name) : base(name)
        {
            LightIntensity = DefaultIntensity;
            LightRadius = DefaultRadius;
            Color = Color.White;
            IsVisible = true;
        }
        
        public Color Color { get; set; }
        
        [Browsable(false)]
        public Vector3 LightPosition { get { return WorldPosition; } }
        
        public float LightRadius { get; set; }
        public float LightIntensity { get; set; }

        public override void PopulateXml(System.Xml.XmlElement element, System.Xml.XmlDocument document)
        {
            if (Color != Color.White)
                XmlHelper.SetAttribute(element, "Color", Color.ToVector4());

            if (LightRadius != DefaultRadius)
                element.SetAttribute("LightRadius", LightRadius.ToString());

            if (LightIntensity != DefaultIntensity)
                element.SetAttribute("LightIntensity", LightIntensity.ToString());

            base.PopulateXml(element, document);
        }
        public override void SetFromXml(System.Xml.XmlElement element)
        {
            if (element.HasAttribute("LightRadius"))
                this.LightRadius = Single.Parse(element.Attributes["LightRadius"].Value);

            if (element.HasAttribute("LightIntensity"))
                this.LightIntensity = Single.Parse(element.Attributes["LightIntensity"].Value);

            if (element.HasAttribute("Color"))
                Color = new Color(XmlHelper.ParseVector4(element.Attributes["Color"]));

            base.SetFromXml(element);
        }

        public override void Update(GameTime dt, DrawInfo drawInfo)
        {
            base.Update(dt, drawInfo);
        }


        #region INodeRenderable Members

        public bool IsVisible { get; set; }

        public void Draw(GameTime dt, DrawInfo drawInfo)
        {
            if (IsVisible)
                drawInfo.PointLights.Add(this);
        }

        #endregion
    }
}
 