using System;
using System.Collections.Generic;
using System.Text;
using Editor.Data;
using System.ComponentModel;
using Microsoft.Xna.Framework;
using GameEngine.Helpers;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using GameEngine.Designer;

namespace GameEngine.Xml
{
    public class PolygonWallNode : PolygonNode, INodeLoadable, INodeRenderable
    {
        [Category("Wall")]
        [DisplayName("Texture")]
        [TypeConverter(typeof(TextureChooser))]
        public string WallTexture
        {
            get { return wallTextureAsset; }
            set
            {
                if (wallTextureAsset == value)
                    return;

                wallTextureAsset = value;
                Dirty();
            }
        }
        [Category("Top")]
        [DisplayName("Texture")]
        [TypeConverter(typeof(TextureChooser))]
        public string TopTexture
        {
            get { return topTextureAsset; }
            set
            {
                if (topTextureAsset == value)
                    return;

                topTextureAsset = value;
                Dirty();
            }
        }


        [Category("Top")]
        [DisplayName("Texture Offset")]
        public Vector2 TopTextureOffset
        {
            get { return topTextureOffset; }
            set
            {
                if (topTextureOffset == value)
                    return;

                topTextureOffset = value;
                Dirty();
            }
        }

        [Category("Top")]
        [DisplayName("Texture Size")]
        public Vector2 TopTextureSize
        {
            get { return topTextureSize; }
            set
            {
                if (topTextureSize == value)
                    return;

                topTextureSize = value;
                Dirty();
            }
        }

        [Category("Top")]
        [DisplayName("Color")]
        public Vector4 TopColor
        {
            get { return topColor; }
            set
            {
                if (topColor == value)
                    return;

                topColor = value;
                Dirty();
            }
        }



        [Category("Wall")]
        [DisplayName("Texture Offset")]
        public Vector2 WallTextureOffset
        {
            get { return wallTextureOffset; }
            set
            {
                if (wallTextureOffset == value)
                    return;

                wallTextureOffset = value;
                Dirty();
            }
        }

        [Category("Wall")]
        [DisplayName("Texture Size")]
        
        public Vector2 WallTextureSize
        {
            get { return wallTextureSize; }
            set
            {
                if (wallTextureSize == value)
                    return;

                wallTextureSize = value;
                Dirty();
            }
        }

        [Category("Wall")]
        [DisplayName("Color")]
        public Vector4 WallColor
        {
            get { return wallColor; }
            set
            {
                if (wallColor == value)
                    return;

                wallColor = value;
                Dirty();
            }
        }

        [Category("PolygonWall")]
        public int Height 
        {
            get { return height; }
            set 
            {
                if (height == value)
                    return;

                height = value;
                Dirty();
            }
        }
        bool wallHasTexture = false, topHasTexture = false;
        private Vector4 topColor = Vector4.One;
        private Vector2 topTextureOffset = Vector2.Zero, topTextureSize = Vector2.Zero;

        private Vector4 wallColor = Vector4.One;
        private Vector2 wallTextureOffset = Vector2.Zero, wallTextureSize = Vector2.Zero;
        public PolygonWallNormals wall;
        public TesselatedPolygonNormals top;
        private Effect fxGBuffer;
        public Texture2D topTexture, wallTexture;
        private string wallTextureAsset = "", topTextureAsset = "";
        private GraphicsDevice device;
        private ContentManager content;
        private GeometryDrawInfo topGdi = new GeometryDrawInfo();
        private GeometryDrawInfo wallGdi = new GeometryDrawInfo();

        private bool isVisible = true;
        private int height = 1;

        [Category("Renderable")]
        public bool IsVisible
        {
            get
            {
                return isVisible;
            }
            set
            {
                isVisible = value;
            }
        }

        public PolygonWallNode(string name) : base(name)
        {

        }

        public override void PopulateXml(System.Xml.XmlElement element, System.Xml.XmlDocument document)
        {
            if (!string.IsNullOrEmpty(TopTexture))
                element.SetAttribute("TopTexture", TopTexture);

            if (TopTextureOffset != Vector2.Zero)
                XmlHelper.SetAttribute(element, "TopTextureOffset", TopTextureOffset);

            if (TopTextureSize != Vector2.Zero)
                XmlHelper.SetAttribute(element, "TopTextureSize", TopTextureSize);

            if (TopColor != Vector4.One)
                XmlHelper.SetAttribute(element, "TopColor", TopColor);
            
            
            if (!string.IsNullOrEmpty(WallTexture))
                element.SetAttribute("WallTexture", WallTexture);
            
            if (WallTextureOffset != Vector2.Zero)
                XmlHelper.SetAttribute(element, "WallTextureOffset", WallTextureOffset);
            
            if (WallTextureSize != Vector2.Zero)
                XmlHelper.SetAttribute(element, "WallTextureSize", WallTextureSize);

            if (WallColor != Vector4.One)
                XmlHelper.SetAttribute(element, "WallColor", WallColor);
            

            if (Height != 1)
                element.SetAttribute("Height", Height.ToString());
            base.PopulateXml(element, document);
        }

        public override void SetFromXml(System.Xml.XmlElement element)
        {
            if (element.HasAttribute("Height"))
                this.Height = Int32.Parse(element.Attributes["Height"].Value);

            if (element.HasAttribute("TopTexture"))
                this.TopTexture = element.Attributes["TopTexture"].Value;

            if (element.HasAttribute("TopTextureOffset"))
                TopTextureOffset = XmlHelper.ParseVector2(element.Attributes["TopTextureOffset"]);

            if (element.HasAttribute("TopTextureSize"))
                TopTextureSize = XmlHelper.ParseVector2(element.Attributes["TopTextureSize"]);

            if (element.HasAttribute("TopColor"))
                TopColor = XmlHelper.ParseVector4(element.Attributes["TopColor"]);
            
            
            if (element.HasAttribute("WallTexture"))
                this.WallTexture = element.Attributes["WallTexture"].Value;
            
            if (element.HasAttribute("WallTextureOffset"))
                WallTextureOffset = XmlHelper.ParseVector2(element.Attributes["WallTextureOffset"]);

            if (element.HasAttribute("WallTextureSize"))
                WallTextureSize = XmlHelper.ParseVector2(element.Attributes["WallTextureSize"]);

            if (element.HasAttribute("WallColor"))
                WallColor = XmlHelper.ParseVector4(element.Attributes["WallColor"]);

            base.SetFromXml(element);
        }

        public override void ResolveDirty()
        {
            UnloadContent();
            LoadContent(content, device);

            base.ResolveDirty();
        }

        public void LoadContent(Microsoft.Xna.Framework.Content.ContentManager content, Microsoft.Xna.Framework.Graphics.GraphicsDevice graphicsDevice)
        {
            device = graphicsDevice;
            this.content = content;

            if (content == null)
                return;

            // load effects
            content.Load(EngineFX.RenderGBuffer, out fxGBuffer);

            wallHasTexture = content.TryLoad(WallTexture, out wallTexture);
            topHasTexture = content.TryLoad(TopTexture, out topTexture);

            if (!wallHasTexture)
            {
                wallTexture = new Texture2D(graphicsDevice, 1, 1);
                wallTexture.SetData(new Color[] { new Color(wallColor)});
            }

            if (!topHasTexture)
            {
                topTexture = new Texture2D(graphicsDevice, 1, 1);
                topTexture.SetData(new Color[] { new Color(topColor) });
            }

            wall = PolygonWallNormals.Generate(device, Points, Height, WallTextureOffset, WallTextureSize, WallColor);
            top = TesselatedPolygonNormals.Generate(device, Points, new Vector3(0, Height, 0), TopTextureOffset, TopTextureSize, Vector4.One);
            
            topGdi.geometry = top;
            topGdi.texture = topTexture;
            topGdi.textureAsset = TopTexture;
            
            wallGdi.geometry = wall;
            wallGdi.texture = wallTexture;
            wallGdi.textureAsset = WallTexture;
        }
 
        public void UnloadContent()
        {
            Util.NullDispose(ref wall);
            Util.NullDispose(ref top);
            
            if(!wallHasTexture)
                Util.NullDispose(ref wallTexture);

            if (!topHasTexture)
                Util.NullDispose(ref topTexture);
        }

        public void Draw(GameTime dt, DrawInfo drawInfo)
        {
            if (wall != null)
            {
                wallGdi.world = World;
                topGdi.world = World;

                drawInfo.Gdis.Add(wallGdi);
                drawInfo.Gdis.Add(topGdi);

                /*

                fxGBuffer.Parameters["World"].SetValue(World);

                fxGBuffer.Parameters["Texture"].SetValue(wallTexture);

                wall.Draw(drawInfo.GraphicsDevice, fxGBuffer);

                fxGBuffer.Parameters["Texture"].SetValue(topTexture);

                top.Draw(drawInfo.GraphicsDevice, fxGBuffer);

                /*wallEffect.CurrentTechnique = wallEffect.Techniques[0];

                EngineFX.SetPolyWallSharedParams(wallEffect, drawInfo);

                wallEffect.Parameters["World"].SetValue(World);

                if (wallTexture != null)
                    drawInfo.GraphicsDevice.Textures[0] = wallTexture;

                wallEffect.Parameters["useTexture"].SetValue(wallTexture != null);

                wall.Draw(drawInfo.GraphicsDevice, wallEffect);

                if (topTexture != null)
                    drawInfo.GraphicsDevice.Textures[0] = topTexture;

                wallEffect.Parameters["useTexture"].SetValue(topTexture != null);

                top.Draw(drawInfo.GraphicsDevice, wallEffect);*/
            }

        }
    }
}
