/*

% Description of my shader.
% Second line of description for my shader.

keywords: material classic

date: YYMMDD

*/

float4x4 WorldViewProj : WorldViewProjection;
float4x4 Bones[59];

texture BasicTexture
<
   string ResourceName = "";
   string UIName =  "Texture";
   string ResourceType = "2D";
>;

sampler TextureSampler : register(s0) = sampler_state
{
	Texture = (BasicTexture);
	MipFilter = Linear;
	MinFilter = Linear;
	MagFilter = Linear;
	AddressU = Wrap;
    AddressV = Wrap;
};

struct VS_OUTPUT 
{
   float4 Position : POSITION0;
   float2 Texture : TEXCOORD0;
};

VS_OUTPUT mainVS(float3 pos : POSITION, float2 texCoord : TEXCOORD0, float4 BoneIndices : BLENDINDICES, float4 BoneWeights : BLENDWEIGHT )
{
	VS_OUTPUT o;

// Blend between the weighted bone matrices.
	float4x4 skinTransform = 0;

	skinTransform += Bones[BoneIndices.x] * BoneWeights.x;
	skinTransform += Bones[BoneIndices.y] * BoneWeights.y;
	skinTransform += Bones[BoneIndices.z] * BoneWeights.z;
	skinTransform += Bones[BoneIndices.w] * BoneWeights.w;

	// Skin the vertex position.
 	float4 weightedposition = mul(float4(pos.xyz, 1.0), skinTransform);

	// transform in screen space
	//OUT.position = mul( mul(weightedposition, View), Projection );
	
	o.Position = mul(weightedposition, WorldViewProj);
	o.Texture = texCoord;
	return o;
}

float4 mainPS(VS_OUTPUT v) : COLOR {
	
	return tex2D(TextureSampler, v.Texture);
}

technique technique0 {
	pass p0 {
		VertexShader = compile vs_3_0 mainVS();
		PixelShader = compile ps_3_0 mainPS();
	}
}
