
float4x4 World;
float4x4 View;
float4x4 Projection;
float specularIntensity = 0;

texture Texture;
sampler diffuseSampler = sampler_state
{
    Texture = (Texture);
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

struct VertexShaderOutputColor
{
    float4 Position : POSITION0;
	float4 Color : COLOR0;
};

struct VertexShaderInput
{
    float4 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
};


VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);

    output.TexCoord = input.TexCoord; 
    
    return output;
}

struct PixelShaderOutput
{
    float4 Color : COLOR0;

};


PixelShaderOutput PixelShaderFunction(VertexShaderOutput input)
{
    PixelShaderOutput output;
    output.Color =  tex2D(diffuseSampler, input.TexCoord); 
    
    return output;
}


technique Decal
{
    pass Pass1
    {
        VertexShader = compile vs_2_0 
        VertexShaderFunction();
        PixelShader = compile ps_2_0 
        PixelShaderFunction();
    }
}
