using System;
using System.Collections.Generic;
using System.Text;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework;
using System.Runtime.Serialization;
using System.IO;
using System.Reflection;
using System.Runtime.Serialization.Formatters.Binary;
using GameEngine;

namespace GamePrototype.GUI.Screens
{
    public class ConfigureControlsScreen : GameScreen
    {
        private const string dataFile = "Gameconfig.dat";
        private const string headLine = "Configure Keyboard";
        private string[] playerTexts = new string[] {"Layout 1","Layout 2" };
        private string[] helpTexts = new string[] { "Use Arrow Keys to Navigate.", "Use Enter Key to Change a Value." };
        private string[] buttonTexts = new string[] {"Save Changes","Back to Main Menu" };
        private const string saveText = "Save Changes?";
        private string[] saveOptions = new string[] {"Yes","No" };    
                
        private const float helpTextScale = 1.2f;
        private const float headLineScale = 2.0f;
        private const float playerTextsScale = 1.4f;
        private Color headLineColor = Color.CornflowerBlue;
        private Color helpTextColor = Color.Gray;
        private const float spaceBetweenHeaders = 5f;
        private const float spaceBetweenDocAndControls = 60;
        private const float spaceBetweenControlsHorizontal = 90;
        private const float spaceFromPlayerTextsToOptions = 20;
        private const float spaceFromControlsToButtons = 20;
        private const float spaceBetweenButtons = 15f;

        private const float buttonsScale = 1.7f;
        private const float controlsScale = 1.4f;
        private const float spaceBetweenControlsVertical = 10f;
        private Color controlsColor = Color.CornflowerBlue;
        private Color controlsSelectedColor = Color.White;
        private Color backGroundColor = Color.Black;
        private Color borderColor = Color.Gray;
        
        private int currentHorizontalIndex = 0;
        private int currentVerticalIndex = 0;
        private int overallIndex = 0;

        private Texture2D texture;

        private PlayerControl playerControl;

        public ConfigureControlsScreen(string id, ScreenController controller, Game game) 
            : base(id, controller, game)
	    {
            
	    }

        public override void Initialize(GraphicsDevice graphics)
        {

        }

        public override void LoadContent(GraphicsDeviceManager graphics)
        {
            texture = new Texture2D(graphics.GraphicsDevice, 2, 2, 1, TextureUsage.None, SurfaceFormat.Vector4);
            Vector4[] data = new Vector4[2 * 2];

            for (int i = 0; i < data.Length; i++)
            {
                data[i] = Vector4.One;    
            }

            texture.SetData(data);
        }

        public override void Update(GameTime gameTime)
        {
            int overallIndexOld = overallIndex;
            int currentHorizontalIndexOld = currentHorizontalIndex;
            int currentVerticalIndexOld = currentVerticalIndex;
                
            DialogueScreen dialogue = (DialogueScreen)this.controller.GetScreen("DialogueScreen");

            if (dialogue.DialogueResult != "")
            {
                string[] temp1 = dialogue.DialogueResult.Split('|');

                if (temp1[0] == "ConfigureControlsScreen")
                {
                    string[] temp2 = temp1[1].Split(':');

                    if (temp2.Length == 2) 
                    {
                        int playerIndex;
                        bool succes = Int32.TryParse(temp2[0].Substring(temp2[0].Length - 2), out playerIndex);
                        playerIndex -= 1;
                        int option = Array.IndexOf(playerControl.Documentation, temp2[1]);

                        if (!succes || (option == -1) || (playerIndex < 0) || playerIndex >= playerControl.Controls.GetLength(0))
                        {
                            dialogue.DialogueResult = "";
                            return;
                        }

                        Keys newKey = (Keys)Enum.Parse(typeof(Keys), temp1[2], true);

                        for (int i = 0; i < playerControl.Controls.GetLength(0); i++)
                        {
                            for (int j = 0; j < playerControl.Controls.GetLength(1); j++)
                            {
                                bool exists = newKey == playerControl.Controls[i, j];
                                                                
                                if (exists)
	                            {
                                    int keysLength = Enum.GetValues(typeof(Keys)).Length;
                                    int notUsed;
                                    
                                    while (exists)
                                    {
                                        Keys randomKey = (Keys)Util.Random.Next(keysLength);

                                        if (randomKey.ToString().Length == 1
                                            && !Int32.TryParse(randomKey.ToString(), out notUsed)
                                            && !DoesKeyExist(randomKey))
                                        {
                                            playerControl.Controls[i, j] = randomKey;
                                            exists = false;
                                        }
                                    }
	                            }                                      
                            }                            
                        }                       
                                                
                        playerControl.Controls[playerIndex, option] = newKey;
                        
                    }
                    else // Save Changes?
                    {
                        if (temp1[1] == saveText)
                        {
                            if (temp1[2] == saveOptions[0]) // Yes
                            {
                                SavePlayerControls();
                                this.controller.SetHiddenScreen("ConfigureControlsScreen");
                                this.controller.SetActiveScreen("MainMenuScreen");
                            }
                            if (temp1[2] == saveOptions[1]) // No
                            {
                                this.controller.SetHiddenScreen("ConfigureControlsScreen");
                                this.controller.SetActiveScreen("MainMenuScreen");
                            }
                        }
                    }                  
                }

                dialogue.DialogueResult = "";
            }


            if (InputState.KeyPressed(Keys.Enter) || InputState.ButtonPressedOnAnyGamePad(Buttons.A))
            {
                if (overallIndex == 0) 
                {
                    string text = "Press New Key for Player " + (currentHorizontalIndex + 1).ToString() + ":" +
                        playerControl.Documentation[currentVerticalIndex];
 
                    this.controller.SetInactiveScreen("ConfigureControlsScreen");
                    dialogue.SetDialogueData(text, new string[]{}, "ConfigureControlsScreen", 0);
                    this.controller.SetActiveScreen("DialogueScreen");        
                }
                else if (overallIndex == 1) // Save Changes
                {
                    SavePlayerControls();
                    this.controller.SetHiddenScreen("ConfigureControlsScreen");
                    this.controller.SetActiveScreen("MainMenuScreen");
                }
                else if (overallIndex == 2) // Back to main menu
                {
                    PlayerControl freshLoaded;
                    LoadPlayerControls(out freshLoaded);

                    if (freshLoaded.IsTheSame(playerControl.Controls)) // Nothing is changed
                    {
                        this.controller.SetHiddenScreen("ConfigureControlsScreen");
                        this.controller.SetActiveScreen("MainMenuScreen");
                    }
                    else
                    {
                        this.controller.SetInactiveScreen("ConfigureControlsScreen");
                        dialogue.SetDialogueData(saveText, saveOptions, "ConfigureControlsScreen", 0);
                        this.controller.SetActiveScreen("DialogueScreen");                        
                    }                    
                }    
            }            

            if (InputState.KeyPressed(Keys.Right) || InputState.ButtonPressedOnAnyGamePad(Buttons.DPadRight))
            {
                if (overallIndex == 0)
                {
                    currentHorizontalIndex = (currentHorizontalIndex == playerControl.Controls.GetLength(0) - 1) ?
                    playerControl.Controls.GetLength(0) - 1 : currentHorizontalIndex + 1;    
                }
            }

            if (InputState.KeyPressed(Keys.Left) || InputState.ButtonPressedOnAnyGamePad(Buttons.DPadLeft))
            {
                if (overallIndex == 0)
                {
                    currentHorizontalIndex = currentHorizontalIndex == 0 ? 0 : currentHorizontalIndex - 1;    
                }               
            }

            if (InputState.KeyPressed(Keys.Up) || InputState.ButtonPressedOnAnyGamePad(Buttons.DPadUp))
            {
                if (overallIndex == 0 && currentVerticalIndex > 0)
                {
                    currentVerticalIndex--;
                }
                else if (overallIndex == 0 && currentVerticalIndex == 0)
                {
                    overallIndex = 2;
                }
                else if (overallIndex == 2)
                {
                    overallIndex = 1;
                }
                else if (overallIndex == 1)
                {
                    overallIndex = 0;
                    currentVerticalIndex = playerControl.Controls.GetLength(1) - 1;
                }
            }

            if (InputState.KeyPressed(Keys.Down) || InputState.ButtonPressedOnAnyGamePad(Buttons.DPadDown))
            {
                if (overallIndex == 0 && currentVerticalIndex < playerControl.Controls.GetLength(1) - 1)
                {
                    currentVerticalIndex++;
                }
                else if (overallIndex == 0 && currentVerticalIndex == playerControl.Controls.GetLength(1) - 1)
                {
                    overallIndex++;
                }
                else if (overallIndex > 0)
                {
                    if (overallIndex == 2)
                    {
                        overallIndex = 0;
                        currentVerticalIndex = 0;
                    }
                    else
                    {
                        overallIndex++;
                    }
                }
            }

            if (InputState.KeyPressed(Keys.Escape))
            {
                PlayerControl freshLoaded;
                LoadPlayerControls(out freshLoaded);

                if (freshLoaded.IsTheSame(playerControl.Controls)) // Nothing is changed
                {
                    this.controller.SetHiddenScreen("ConfigureControlsScreen");
                    this.controller.SetActiveScreen("MainMenuScreen");
                }
                else
                {
                    this.controller.SetInactiveScreen("ConfigureControlsScreen");
                    dialogue.SetDialogueData(saveText, saveOptions, "ConfigureControlsScreen", 0);
                    this.controller.SetActiveScreen("DialogueScreen");
                }
            }
            
            if (!(overallIndex == overallIndexOld && currentHorizontalIndex == currentHorizontalIndexOld &&
                currentVerticalIndex == currentVerticalIndexOld))
            {
                AudioManager.PlaySoundEffect(Game1.MENU_CLICK, Game1.MENU_CLICK_VOLUME);         
            }                        
        }

        public override void Draw(GameTime gameTime, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            graphics.Clear(backGroundColor);

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
                        
            DrawHeader(spriteBatch, spriteFont);
            DrawPlayerControls(spriteBatch, spriteFont);
            DrawButtons(spriteBatch, spriteFont);
            
            spriteBatch.End();            
        }

        private void DrawHeader(SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            float x = GUIhelper.GetHorisontalCenter(headLine, spriteFont, Game, headLineScale);
            float y = GUIhelper.GetVerticalCenter(headLine, spriteFont, Game, headLineScale) * 0.15f;
            Vector2 headLinePosition = new Vector2(x, y);

            spriteBatch.DrawString(spriteFont, headLine, headLinePosition, headLineColor, 0, Vector2.Zero,
                headLineScale, SpriteEffects.None, 0);

            float headlineHeight = spriteFont.MeasureString(headLine).Y * headLineScale;
            float helpTextHeight = spriteFont.MeasureString(helpTexts[0]).Y * helpTextScale;

            y += headlineHeight +  2 * spaceBetweenHeaders;

            for (int i = 0; i < helpTexts.Length; i++)
            {
                spriteBatch.DrawString(spriteFont, helpTexts[i],
                new Vector2(GUIhelper.GetHorisontalCenter(helpTexts[i], spriteFont, Game, helpTextScale), y),
                    helpTextColor, 0, Vector2.Zero, helpTextScale, SpriteEffects.None, 0);

                y += helpTextHeight + spaceBetweenHeaders;
            }
        }

        private void DrawPlayerControls(SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            float largestDocumentation = 0;

            for (int i = 0; i < playerControl.Documentation.Length; i++)
            {
                float j = spriteFont.MeasureString(playerControl.Documentation[i]).X * controlsScale;

                largestDocumentation = j > largestDocumentation ? j : largestDocumentation;
            }

            float largestKey = 0;

            for (int i = 0; i < playerControl.Controls.GetLength(0); i++)
            {
                for (int j = 0; j < playerControl.Controls.GetLength(1); j++)
                {
                    float k = spriteFont.MeasureString(playerControl.Controls[i, j].ToString()).X * controlsScale;

                    largestKey = k > largestKey ? k : largestKey;
                }
            }

            float controlsWidth = largestDocumentation + spaceBetweenDocAndControls + largestKey;

            float xPos = (Game.Window.ClientBounds.Width - (controlsWidth * 2 + spaceBetweenControlsHorizontal)) / 2;
            float yPos = Game.Window.ClientBounds.Height * 0.25f;

            float player1xPos = (xPos + controlsWidth / 2) - (spriteFont.MeasureString(playerTexts[0]).X * playerTextsScale) / 2;
            float player2xPos = player1xPos + controlsWidth + spaceBetweenControlsHorizontal;

            spriteBatch.DrawString(spriteFont, playerTexts[0], new Vector2(player1xPos, yPos), helpTextColor, 0, Vector2.Zero, 
                playerTextsScale, SpriteEffects.None, 0);
            spriteBatch.DrawString(spriteFont, playerTexts[1], new Vector2(player2xPos, yPos), helpTextColor, 0, Vector2.Zero,
                playerTextsScale, SpriteEffects.None, 0);

            yPos += (spriteFont.MeasureString(playerTexts[0]).Y * playerTextsScale) + spaceFromPlayerTextsToOptions;

            for (int i = 0; i < playerControl.Controls.GetLength(1); i++)
            {
                float tempX = xPos;
                
                for (int j = 0; j < playerControl.Controls.GetLength(0); j++)
                {
                    Color itemColor = (i == currentVerticalIndex && i < playerControl.Controls.GetLength(1) && 
                        j == currentHorizontalIndex && overallIndex == 0) ? controlsSelectedColor : controlsColor;

                    // Player texts 
                    spriteBatch.DrawString(spriteFont, playerControl.Documentation[i], new Vector2(tempX, yPos),
                        helpTextColor, 0, Vector2.Zero, controlsScale, SpriteEffects.None, 0);

                    // Player controls
                    spriteBatch.DrawString(spriteFont, playerControl.Controls[j, i].ToString(), new Vector2
                        (tempX + largestDocumentation + spaceBetweenDocAndControls, yPos), itemColor, 0, Vector2.Zero,
                        controlsScale, SpriteEffects.None, 0);

                    tempX += controlsWidth + spaceBetweenControlsHorizontal;
                }
                
                yPos += (spriteFont.MeasureString(playerControl.Documentation[0]).Y * controlsScale) + spaceBetweenControlsVertical;
            }
        }

        private void DrawButtons(SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            float yPos = Game.Window.ClientBounds.Height * 0.30f;
            yPos += spriteFont.MeasureString(playerTexts[0]).Y * playerTextsScale;
            yPos += spaceFromPlayerTextsToOptions;
            yPos += spriteFont.MeasureString(playerControl.Documentation[0]).Y * controlsScale * playerControl.Documentation.Length;
            yPos += (playerControl.Documentation.Length - 1) * spaceBetweenControlsVertical;
            yPos += spaceFromControlsToButtons;

            float xPos = GUIhelper.GetHorisontalCenter(buttonTexts[0], spriteFont, Game, buttonsScale);

            for (int i = 0; i < buttonTexts.Length; i++)
            {
                Color itemColor = overallIndex == (i + 1) ? controlsSelectedColor : controlsColor;

                spriteBatch.DrawString(spriteFont, buttonTexts[i], new Vector2(xPos, yPos), itemColor, 0, Vector2.Zero, buttonsScale, SpriteEffects.None, 0);

                yPos += (spriteFont.MeasureString(buttonTexts[i]).Y * buttonsScale) + spaceBetweenButtons;
            }
        }

        public void LoadPlayerControls()
        {
            if (!File.Exists(GetDataFile()))
            {
                playerControl = new PlayerControl();
                playerControl.SetStandard();
            }
            else
            {
                FileStream fs = null;

                try
                {
                    IFormatter f = new BinaryFormatter();
                    fs = new FileStream(GetDataFile(), FileMode.Open);
                    playerControl = (PlayerControl)f.Deserialize(fs);
                }
                catch (Exception)
                {
                    playerControl = new PlayerControl();
                    playerControl.SetStandard();
                }

                if (fs != null)
                {
                    fs.Dispose();
                }
            }
        }

        public void LoadPlayerControls(out PlayerControl control)
        {
            if (!File.Exists(GetDataFile()))
            {
                control = new PlayerControl();
                control.SetStandard();
            }
            else
            {
                FileStream fs = null;

                try
                {
                    IFormatter f = new BinaryFormatter();
                    fs = new FileStream(GetDataFile(), FileMode.Open);
                    control = (PlayerControl)f.Deserialize(fs);
                }
                catch (Exception)
                {
                    control = new PlayerControl();
                    control.SetStandard();
                }

                if (fs != null)
                {
                    fs.Dispose();
                }
            }
        }

        public void SavePlayerControls()
        {
            if (playerControl == null)
            {
                playerControl = new PlayerControl();
            }

            IFormatter f = new BinaryFormatter();
            FileStream fs = new FileStream(GetDataFile(), FileMode.Create);
            f.Serialize(fs, playerControl);
            fs.Dispose();
        }

        private string GetDataFile()
        {
            return Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location) + Path.DirectorySeparatorChar + Path.DirectorySeparatorChar + dataFile;
        }

        public PlayerControl GetControls()
        {
            LoadPlayerControls();
            return playerControl;
        }

        private bool DoesKeyExist(Keys key)
        {
            foreach (Keys k in playerControl.Controls)
            {
                if (k == key)
                {
                    return true;
                }    
            }

            return false;
        }
    }
}
