﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework;
using GameEngine.LevelGeneration;
using GameEngine.TextureGeneration;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace GamePrototype.GUI.Screens
{
    public class LeninDebug : GameScreen
    {
        private Texture2D tile;
        private Texture2D edge;
        private LevelGenerator lg;
        LevelTile[,] theLevel;
        int levelWidth, levelHeight;

        private int AverageTilecount = 75;
        private int AllowedDeviation = 25;
        private int scale = 4;

        private BloodGenerator bg;

        public LeninDebug(string id, ScreenController controller, Game game)
            : base(id, controller, game)
        {
            lg = new LevelGenerator(AverageTilecount, AllowedDeviation);
            theLevel = lg.GetLevelTileArray(out levelWidth, out levelHeight);
        }

        public override void LoadContent(GraphicsDeviceManager graphics)
        {
            tile = new Texture2D(graphics.GraphicsDevice, 3 * scale, 3 * scale, 1, TextureUsage.None, SurfaceFormat.Vector4);
            Vector4[] data = new Vector4[(3 * scale) * (3 * scale)];
            for (int i = 0; i < data.Length; i++)
                data[i] = Vector4.One;
            tile.SetData(data);

            edge = new Texture2D(graphics.GraphicsDevice, scale, scale, 1, TextureUsage.None, SurfaceFormat.Vector4);
            Vector4[] data2 = new Vector4[scale * scale];
            for (int i = 0; i < data2.Length; i++)
                data2[i] = Vector4.One;
            edge.SetData(data2);

            bg = new BloodGenerator(graphics.GraphicsDevice);
        }

        public override void Update(GameTime gameTime)
        {
            if (InputState.KeyPressed(Keys.Escape))
            {
                Game.Exit();
            }

            if (InputState.KeyPressed(Keys.S)) // Start game (debug purpose)
            {
                this.controller.SetHiddenScreen("GenerateLevelScreen");
                this.controller.SetActiveScreen("MainGameScreen");
            }

            if (InputState.KeyPressed(Keys.R))
            {
                lg.PositionPoints(4);

                List<LevelTile> res = new List<LevelTile>();

                foreach (LevelTile tile in lg.CreatedTiles)
                {
                    if (tile.IsPointOfInterest == true)
                    {
                        res.Add(tile);

                    }
                }

            }

            if (InputState.KeyPressed(Keys.Space))
            {
                lg.ConstructLevel(AverageTilecount, AllowedDeviation);
                lg.PositionPoints(4);
                theLevel = lg.GetLevelTileArray(out levelWidth, out levelHeight);
                //bg.LiquidAmount = 4000;// BloodGenerator.random.Next(80, 2560);
                //bg.Slope = 0f;// (float)BloodGenerator.random.NextDouble() * MathHelper.PiOver2;
                //bg.Generate();
                //texture = bg.GetTexture();
            }
        }

        public override void Draw(GameTime gameTime, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            graphics.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();
            
            int levelLeft = (Game.Window.ClientBounds.Width / 2) - levelWidth * scale;
            int levelTop = (Game.Window.ClientBounds.Height / 2) - levelHeight * scale;

            int edgeCount = 0;
            int[] typeCounts = new int[5];

            for (int i = 0; i < levelWidth; i++)
            {
                for (int j = 0; j < levelHeight; j++)
                {
                    if (theLevel[i, j] == null)
                    {
                        spriteBatch.Draw(tile, new Vector2(levelLeft + scale * (i * 3), levelTop + scale * (j * 3)), Color.Black);
                    }
                    else
                    {

                        if (theLevel[i, j].IsPointOfInterest == true)
                        {
                            spriteBatch.Draw(tile, new Vector2(levelLeft + scale * (i * 3), levelTop + scale * (j * 3)), Color.Red);
                        }
                        else
                        {
                            spriteBatch.Draw(tile, new Vector2(levelLeft + scale * (i * 3), levelTop + scale * (j * 3)), Color.White);
                        }

                        spriteBatch.Draw(edge, new Vector2(levelLeft + scale * ((i * 3) + 1), levelTop + scale * ((j * 3) + 1)), Color.Blue);
                        if (theLevel[i, j].TopEdge == EdgeState.Edge)
                            spriteBatch.Draw(edge, new Vector2(levelLeft + scale * ((i * 3) + 1), levelTop + scale * ((j * 3))), Color.Blue);
                        if (theLevel[i, j].RightEdge == EdgeState.Edge)
                            spriteBatch.Draw(edge, new Vector2(levelLeft + scale * ((i * 3) + 2), levelTop + scale * ((j * 3) + 1)), Color.Blue);
                        if (theLevel[i, j].BottomEdge == EdgeState.Edge)
                            spriteBatch.Draw(edge, new Vector2(levelLeft + scale * ((i * 3) + 1), levelTop + scale * ((j * 3) + 2)), Color.Blue);
                        if (theLevel[i, j].LeftEdge == EdgeState.Edge)
                            spriteBatch.Draw(edge, new Vector2(levelLeft + scale * (i * 3), levelTop + scale * ((j * 3) + 1)), Color.Blue);

                        switch (theLevel[i, j].TileType)
                        {
                            case TileType.End:
                                edgeCount += 1;
                                typeCounts[0]++;
                                break;
                            case TileType.Hallway:
                                edgeCount += 2;
                                typeCounts[1]++;
                                break;
                            case TileType.Corner:
                                edgeCount += 2;
                                typeCounts[2]++;
                                break;
                            case TileType.Tintersection:
                                edgeCount += 3;
                                typeCounts[3]++;
                                break;
                            case TileType.Crossing:
                                edgeCount += 4;
                                typeCounts[4]++;
                                break;
                            default:
                                break;
                        }
                    }
                }
            }

            float fontSize = .7f;
            int widthsep = Game.Window.ClientBounds.Width / 9;
            spriteBatch.DrawString(spriteFont, "Tilecount: " + lg.CreatedTiles.Count, new Vector2(15, Game.Window.ClientBounds.Height - 30), Color.White, 0, Vector2.Zero, fontSize, SpriteEffects.None, 0);
            spriteBatch.DrawString(spriteFont, "Edgecount: " + edgeCount, new Vector2(widthsep, Game.Window.ClientBounds.Height - 30), Color.White, 0, Vector2.Zero, fontSize, SpriteEffects.None, 0);
            spriteBatch.DrawString(spriteFont, "Tile type counts: ", new Vector2(widthsep * 3, Game.Window.ClientBounds.Height - 30), Color.White, 0, Vector2.Zero, fontSize, SpriteEffects.None, 0);
            spriteBatch.DrawString(spriteFont, "Ends: " + typeCounts[0], new Vector2(widthsep * 4, Game.Window.ClientBounds.Height - 30), Color.White, 0, Vector2.Zero, fontSize, SpriteEffects.None, 0);
            spriteBatch.DrawString(spriteFont, "Hallways: " + typeCounts[1], new Vector2(widthsep * 5, Game.Window.ClientBounds.Height - 30), Color.White, 0, Vector2.Zero, fontSize, SpriteEffects.None, 0);
            spriteBatch.DrawString(spriteFont, "Corners: " + typeCounts[2], new Vector2(widthsep * 6, Game.Window.ClientBounds.Height - 30), Color.White, 0, Vector2.Zero, fontSize, SpriteEffects.None, 0);
            spriteBatch.DrawString(spriteFont, "T-intersections: " + typeCounts[3], new Vector2(widthsep * 7, Game.Window.ClientBounds.Height - 30), Color.White, 0, Vector2.Zero, fontSize, SpriteEffects.None, 0);
            spriteBatch.DrawString(spriteFont, "Crossings: " + typeCounts[4], new Vector2(widthsep * 8, Game.Window.ClientBounds.Height - 30), Color.White, 0, Vector2.Zero, fontSize, SpriteEffects.None, 0);
            /*
            

            if (texture != null)
            {
                int xPos = (game.Window.ClientBounds.Width / 2) - (scale * texture.Width / 2);
                int yPos = (game.Window.ClientBounds.Height / 2) - (scale * texture.Height / 2);
                spriteBatch.Draw(texture, new Vector2(xPos, yPos), null, Color.White, 0f, Vector2.Zero, (float)scale, SpriteEffects.None, 0);
            }
            else
            {
                bg.LiquidAmount = 2560;
                bg.Generate();
                texture = bg.GetTexture();
            }
            spriteBatch.DrawString(spriteFont, "Space for more splat! - Texture size: " + texture.Width + "x" + texture.Height, new Vector2(15, game.Window.ClientBounds.Height - 30), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0);
            
            */
            spriteBatch.End();
        }

        public override void Initialize(GraphicsDevice graphics)
        {

        }
    }
}
