using System;
using System.Collections.Generic;
using System.Text;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace GamePrototype.GUI.Screens
{
    public class MainMenuScreen : GameScreen   
    {
        private string[] menuItems = new string[] { "Start New Game", "Configure Keyboard", "Configure Graphics", "Exit Game" };
        private const int spaceBetweenMenuItems = 60;
        private const float textScale = 2.0f;
        private Color menuItemColor = Color.CornflowerBlue;
        private Color menuItemSelectedColor = Color.White;
        private Color backGroundColor = Color.Black;
        private int currentMenuIndex = 0;
        
        public MainMenuScreen(string id, ScreenController controller, Game game) 
            : base(id, controller, game)
	    {

	    }

        public override void LoadContent(GraphicsDeviceManager graphics)
        {
            
        }
        
        public override void Update(GameTime gameTime)
        {
            int menuIndex = currentMenuIndex;

            DialogueScreen dialogue = (DialogueScreen)this.controller.GetScreen("DialogueScreen");

            if (dialogue.DialogueResult == "MainMenuScreen|Really Exit?|Yes")
            {
                Game.Exit();
            }

            if (InputState.KeyPressed(Keys.Up) || InputState.ButtonPressedOnAnyGamePad(Buttons.DPadUp)) 
            { 
                currentMenuIndex = (currentMenuIndex == 0) ? menuItems.Length - 1 : currentMenuIndex - 1;
            }

            if (InputState.KeyPressed(Keys.Down) || InputState.ButtonPressedOnAnyGamePad(Buttons.DPadDown)) 
            { 
                currentMenuIndex = (currentMenuIndex = currentMenuIndex + 1) % menuItems.Length; 
            }

            if (InputState.KeyPressed(Keys.Escape) || InputState.ButtonPressedOnAnyGamePad(Buttons.B))
            {
                this.controller.SetHiddenScreen("MainMenuScreen");
                dialogue.SetDialogueData("Really Exit?", new string[] { "Yes", "No" }, "MainMenuScreen", 1);
                this.controller.SetActiveScreen("DialogueScreen");
            }

            if ((InputState.KeyPressed(Keys.Enter) && !InputState.IsKeyDown(Keys.LeftAlt)) || InputState.ButtonPressedOnAnyGamePad(Buttons.A))
            {
                switch (currentMenuIndex)
                {
                    case 0: // Start Game
                        this.controller.SetHiddenScreen("MainMenuScreen");
                        this.controller.SetActiveScreen("SetupGameScreen");
                        break;
                    case 1: // Configure Controls
                        ConfigureControlsScreen controls = (ConfigureControlsScreen)this.controller.GetScreen("ConfigureControlsScreen");
                        controls.LoadPlayerControls();
                        this.controller.SetHiddenScreen("MainMenuScreen");
                        this.controller.SetActiveScreen("ConfigureControlsScreen");
                        break;
                    case 2: // Configure Graphics
                        ConfigureGraphicsOptionsScreen graphicsOptions = (ConfigureGraphicsOptionsScreen)this.controller.GetScreen("ConfigureGraphicsOptionsScreen");
                        graphicsOptions.AboutToShow();
                        this.controller.SetHiddenScreen("MainMenuScreen");
                        this.controller.SetActiveScreen("ConfigureGraphicsOptionsScreen");
                        break;
                    case 3: // Exit Game
                        this.controller.SetInactiveScreen("MainMenuScreen");
                        dialogue.SetDialogueData("Really Exit?", new string[] { "Yes", "No" }, "MainMenuScreen", 1);
                        this.controller.SetActiveScreen("DialogueScreen");
                        break;
                    default:
                        break;
                }       
            }

            if (currentMenuIndex != menuIndex )
            {
                AudioManager.PlaySoundEffect(Game1.MENU_CLICK, Game1.MENU_CLICK_VOLUME);    
            }            
        }

        public override void Draw(GameTime gameTime, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            graphics.Clear(backGroundColor);

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
                        
            Vector2 menuPosition = new Vector2(GUIhelper.GetHorisontalCenter(menuItems[0], spriteFont, Game, textScale),
                (Game.Window.ClientBounds.Height / 2) - ((menuItems.Length - 1) * spaceBetweenMenuItems));
                    
            Color itemColor;

            for (int i = 0; i < menuItems.Length; i++)
            {
                itemColor = (i == currentMenuIndex) ? menuItemSelectedColor : menuItemColor;
                spriteBatch.DrawString(
                    spriteFont, menuItems[i], menuPosition, itemColor, 0, Vector2.Zero, textScale, SpriteEffects.None, 0);
                menuPosition.Y += spaceBetweenMenuItems;
            }

            spriteBatch.End();
        }

        public override void Initialize(GraphicsDevice graphics)
        {

        }
    }
}
