﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine.Physics;
using GameEngine.Helpers;
using Microsoft.Xna.Framework;
using GameEngine.TextureGeneration;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using GameEngine;

namespace GamePrototype.GameLogic
{
    public class BloodEmitter : ParticleSystem
    {
        public static readonly float SPEED = 0.1f;
        public static readonly int DURATION = 2000;
                
        private Vector3 newPosition;

        int splatterPointer = 0;
        int splatterCounter = 0;
        BloodSplatter[] splatter = new BloodSplatter[50];
        QuadNormal decalQuad;
        Effect decalEffect;
        BloodTextures bloodTextures;
        Texture2D clear;
        public void Load(ContentManager content, GraphicsDevice graphics)
        {
            // particles 
            decalQuad = QuadNormal.GenerateXZ(graphics, 100, 0f, Vector2.Zero, Vector2.Zero, Vector4.One);
            decalEffect = content.Load<Effect>(@"EngineFX\DecalGBuffer");
            bloodTextures = new BloodTextures(graphics);
            bloodTextures.GenerateTextures(0, 4, 10, 10, 10, 10);
            clear = new Texture2D(graphics, 1, 1);
            clear.SetData(new Color[] { Color.TransparentBlack });

        }

        public void DrawSplatter(DrawInfo drawInfo)
        {
            drawInfo.GraphicsDevice.VertexDeclaration = decalQuad.decl;
            drawInfo.GraphicsDevice.RenderState.AlphaBlendEnable = true;
            drawInfo.GraphicsDevice.RenderState.SeparateAlphaBlendEnabled = false;
            drawInfo.GraphicsDevice.RenderState.AlphaBlendOperation = BlendFunction.Add;
            drawInfo.GraphicsDevice.RenderState.SourceBlend = Blend.SourceAlpha;
            drawInfo.GraphicsDevice.RenderState.DestinationBlend = Blend.InverseSourceAlpha;


            decalEffect.Parameters["View"].SetValue(drawInfo.Camera.View);
            decalEffect.Parameters["Projection"].SetValue(drawInfo.Camera.Projection);

            for (int i = 0; i < splatter.Length; i++)
            {
                if (splatter[i].InUse)
                {
                    decalEffect.Parameters["Texture"].SetValue(splatter[i].Texture);
                    decalEffect.Parameters["World"].SetValue(splatter[i].World);
                    decalQuad.Draw(drawInfo.GraphicsDevice, decalEffect);
                    
                    //decalEffect.Parameters["Texture"].SetValue(clear);
                    
                    /*drawInfo.GraphicsDevice.RenderState.AlphaBlendOperation = BlendFunction.Add;
                    drawInfo.GraphicsDevice.RenderState.AlphaFunction = CompareFunction.Always;
                    drawInfo.GraphicsDevice.RenderState.SourceBlend = Blend.Zero;
                    drawInfo.GraphicsDevice.RenderState.DestinationBlend = Blend.One;
                    
                    /*drawInfo.GraphicsDevice.RenderState.SeparateAlphaBlendEnabled = true;
                    drawInfo.GraphicsDevice.RenderState.AlphaSourceBlend = Blend.One;
                    drawInfo.GraphicsDevice.RenderState.AlphaDestinationBlend = Blend.One;*/
                    //decalQuad.Draw(drawInfo.GraphicsDevice, decalEffect);
                }
            }


            drawInfo.GraphicsDevice.RenderState.AlphaBlendEnable = false;
        }


        public BloodEmitter(int maxParticles)
            : base(maxParticles)
        {
        }

        protected override void OnParticleUpdate(ParticleSystemParticle p, GameTime time)
        {
            p.Direction += new Vector3(0, -50, 0) * time.ElapsedSeconds();
            
            base.OnParticleUpdate(p, time);
            if (p.Position.Y <= 0)
            {
                Kill(p);
                if (splatterCounter == 0)
                {
                    splatter[splatterPointer].InUse = true;
                    switch (Util.Random.Next(3))
                    {
                        case 0:
                            splatter[splatterPointer].Texture = bloodTextures.GetRandomTexture(BloodAmount.OneLiter);
                            break;
                        case 1:
                            splatter[splatterPointer].Texture = bloodTextures.GetRandomTexture(BloodAmount.TwoLiter);
                            break;
                        case 2:
                            splatter[splatterPointer].Texture = bloodTextures.GetRandomTexture(BloodAmount.HalfLiter);
                            break;
                    }

                    splatter[splatterPointer].World = Matrix.CreateScale((float)splatter[splatterPointer].Texture.Width / 100f, 1, (float)splatter[splatterPointer].Texture.Height / 100f) * Matrix.CreateTranslation(new Vector3(p.Position.X, splatterPointer * 0.01f + 0.01f, p.Position.Z));
                    splatter[splatterPointer].SpawnTime = (float)time.TotalGameTime.TotalSeconds;
                    splatterPointer = (splatterPointer + 1) % splatter.Length;
                }
                splatterCounter++;
                if (splatterCounter == 50)
                    splatterCounter = 0;
            }
        }

        public void DoFragments(Vector2 impact, Vector2 fromDirection, float height, int fragmentCount)
        {
            newPosition = new Vector3(impact.X, height, impact.Y); 
            
            for (int i = 0; i < fragmentCount; i++)
            {
                ParticleSystemParticle p = GetFreeParticle();

                if (p != null)
                {

                    p.Position = newPosition;
                    p.MaxLifeTime = DURATION;
                    p.Speed = SPEED;

                    p.Direction = new Vector3((float)Util.Random.NextDouble() * 15f + fromDirection.X - 7.5f,
                                              (float)Util.Random.NextDouble() * 15f - 7.5f,
                                              (float)Util.Random.NextDouble() * 15f + fromDirection.Y - 7.5f);

                    //p.Direction *= (float)(random.Next(2) - 1);
                
                }
            }
        }
    }
}
