﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine.Helpers;
using Microsoft.Xna.Framework;
using GameEngine.Xml;
using GameEngine.Physics;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework.Input;
using GameEngine;

namespace GamePrototype.GameLogic
{
    public class Enemy : DynamicCircleNode
    {
        public float hitpoints;
        protected Vector2 lastHitInDirection;
        public bool Dead = false;

        public Player CurrentTarget { get; set; }

        public bool IsInUse { get; set; }

        public bool DisableTic; 

        protected Vector3 nextWaypoint = Vector3.Zero;
        public Vector3 moveDirection = Vector3.UnitZ;

        protected float targetDirection = 0;
        public float currentDirection = 0;

        protected float moveSpeed = 300f;
        protected float rotationSpeed = (float)MathHelper.Pi;

        public int TicCounter { get; set; }

        public Enemy(string id, int hitpoints, float moveSpeed) : base(id)
        {
            this.hitpoints = hitpoints;
            this.moveSpeed = moveSpeed;
            CurrentTarget = null;
            TicCounter = 0;
        }

        public void SetNextWaypoint(Vector3 target)
        {
            nextWaypoint = target;
            Vector3 toWayPoint = nextWaypoint - Position;
            UpdateTargetDirection();
        }

        private void UpdateTargetDirection()
        {
            Vector3 toWayPoint = nextWaypoint - Position;
            if (toWayPoint != Vector3.Zero)
            {
                targetDirection =  MathHelper.WrapAngle((float)Math.Atan2(-toWayPoint.Z, toWayPoint.X));
            }
        }




        public Vector2 Position2D { get { return new Vector2(Position.X, Position.Z); } set { Position.X = value.X; Position.Z = value.Y;} }

        public override void Update(GameTime gameTime, DrawInfo di)
        {
            float timeStep = gameTime.ElapsedSeconds();

            if (Dead)
            {
                return;
            }
            base.Update(gameTime, di);


            if (!DisableTic)
            {

                if (targetDirection != currentDirection)
                {
                    float rotationStep = rotationSpeed * timeStep;

                    UpdateTargetDirection();

                    float neededRotation = MathHelper.WrapAngle(targetDirection - currentDirection);

                    if (Math.Abs(neededRotation) > rotationStep)
                    {
                        float angle = Math.Sign(neededRotation) * rotationStep;
                        Quaternion rotateBy = Quaternion.CreateFromAxisAngle(Vector3.UnitY, angle);
                        moveDirection = Vector3.Transform(moveDirection, rotateBy);
                        Rotation = Quaternion.Concatenate(Rotation, rotateBy);
                        currentDirection = MathHelper.WrapAngle(currentDirection + angle);
                    }
                    else
                    {
                        moveDirection = nextWaypoint - Position;
                        if (moveDirection != Vector3.Zero)
                        {
                            Rotation = Quaternion.CreateFromAxisAngle(Vector3.UnitY, (float)Math.Atan2(-moveDirection.Z, moveDirection.X) + MathHelper.PiOver2);
                            moveDirection.Normalize();
                            currentDirection = targetDirection;
                        }
                        else
                        {
                            moveDirection = Vector3.UnitX;
                        }
                    }
                }

                if (nextWaypoint != Position)
                {

                    Vector3 toWayPoint = nextWaypoint - Position;
                    float moveDist = moveSpeed * timeStep;
                    if (toWayPoint.LengthSquared() > moveDist * moveDist)
                    {
                        Position += moveDirection * moveDist;
                    }
                    else
                    {
                        Position += toWayPoint;
                    }
                }
            }
        }

        public virtual void Reset()
        {
            Dead = false;
            this.PassThrough = false;
        }

        public void TakeDamage(int damage, int fromPlayerNumber, bool isZombieCannon, Vector2 fromDirection)
        {
            this.hitpoints -= damage;
            this.lastHitInDirection = fromDirection;

            if (hitpoints <= 0)
            {
                if (isZombieCannon)
	            {
                    GameStatistics.PlayerStats[fromPlayerNumber].ZombieCannonKills++;    
	            }
                else // It must be a normal zombie
                {
                    if ((this as ZombieBear).superFactor >= 1.45f)
                    {
                        GameStatistics.PlayerStats[fromPlayerNumber].SuperZombieKills++;   
                    }
                    else
                    {
                        GameStatistics.PlayerStats[fromPlayerNumber].ZombieKills++;
                    }                                        
                }

                        
            }
        }                
    }
}
