﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using GameEngine.ScreenSystem;

namespace GamePrototype.GameLogic
{
    public class PlayerStats
    {
        public int ZombieKills = 0;
        public int SuperZombieKills = 0;
        public int BulletsFired = 0;
        public int BulletsHit = 0;
        public int ZombieCannonKills = 0;
    }

    public class CommonStats
    {
        public int ZombieTotals = 0;
        public int DamageFromZombies = 0;
        //public int CommonStat3 = 67;
    }
    
    public static class GameStatistics
    {
        public static CommonStats CommonStats = new CommonStats();
        public static PlayerStats[] PlayerStats;

        private static readonly int statsScreenWidth = 950;
        private static readonly int statsScreenHeight = 550;

        public static void Reset(int numberOfPlayers)
        {
            CommonStats = new CommonStats();
            PlayerStats = new PlayerStats[numberOfPlayers];

            for (int i = 0; i < PlayerStats.Length; i++)
            {
                PlayerStats[i] = new PlayerStats();
            }
        }
        
        public static void Draw(GameTime gameTime, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont, Texture2D dotTexture)
        {
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);

            int xPos = (graphics.Viewport.Width - statsScreenWidth) / 2;
            int yPos = (graphics.Viewport.Height - statsScreenHeight) / 2;
            
            // Draw background
            int borderWidth = 2;
            spriteBatch.Draw(dotTexture, new Rectangle(xPos - borderWidth, yPos - borderWidth, statsScreenWidth + 
                2 * borderWidth, statsScreenHeight + 2 * borderWidth), Color.CornflowerBlue); // Border
            spriteBatch.Draw(dotTexture, new Rectangle(xPos, yPos, statsScreenWidth, statsScreenHeight), Color.Black); // Background
            
            // Draw title
            string titleText = "Level Statistics";
            float titleTextScale = 1.8f;
            int titlePadding = 20;
            yPos += titlePadding;
            spriteBatch.DrawString(spriteFont, titleText, new Vector2(
                GUIhelper.GetHorisontalCenter(titleText, spriteFont, graphics, titleTextScale), yPos), Color.CornflowerBlue, 0,
                Vector2.Zero, titleTextScale, SpriteEffects.None, 0);
                        
            // Draw common stats
            yPos += (int)(spriteFont.MeasureString(titleText).Y * titleTextScale);
            int paddingFromTitle = 20;
            int commonStatsEnd = DrawCommonStats(xPos, yPos + paddingFromTitle, graphics, spriteBatch, spriteFont, dotTexture);
            int spaceFromCommonToPlayerStats = 20;

            int statsY = DrawPlayerStats(xPos, commonStatsEnd + spaceFromCommonToPlayerStats, graphics, spriteBatch, spriteFont, dotTexture);
            
            string continueText = "Press Enter or Gamepad StartButton to Continue...";
            float continueScale = 1.4f;
            spriteBatch.DrawString(spriteFont, continueText, new Vector2(
                GUIhelper.GetHorisontalCenter(continueText, spriteFont, graphics, continueScale), statsY + 10), Color.CornflowerBlue, 0,
                Vector2.Zero, continueScale, SpriteEffects.None, 0);

            spriteBatch.End();
        }

        private static int DrawCommonStats(int xPos, int yPos, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont, Texture2D dotTexture)
        {
            Color commonStatsTextColor = Color.Gray;
            Color commonStatsValueColor = Color.Red;
            int space = 40; // between stats vertically
            int padding = 200;
            int xStart = xPos;
            int xEnd = statsScreenWidth - padding + 30;

            Vector2 statText = new Vector2(xStart, yPos);
            Vector2 statValue = new Vector2(xEnd, yPos);

            spriteBatch.DrawString(spriteFont, "Total Zombies", statText, commonStatsTextColor);
            spriteBatch.DrawString(spriteFont, CommonStats.ZombieTotals.ToString(), statValue, commonStatsValueColor);

            statText.Y += space;
            statValue.Y += space;

            spriteBatch.DrawString(spriteFont, "Total Damage From Zombies", statText, commonStatsTextColor);
            spriteBatch.DrawString(spriteFont, CommonStats.DamageFromZombies.ToString(), statValue, commonStatsValueColor);

            statText.Y += space;
            statValue.Y += space;

            //spriteBatch.DrawString(spriteFont, "Stat 3:", statText, commonStatsTextColor);
            //spriteBatch.DrawString(spriteFont, CommonStats.CommonStat3.ToString(), statValue, commonStatsValueColor);

            //statText.Y += space;
            //statValue.Y += space;
            

            return (int)statValue.Y;
        }

        private static int DrawPlayerStats(int xPos, int yPos, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont, Texture2D dotTexture)
        {
            int deltaX = statsScreenWidth / (PlayerStats.Length + 1);
            int startXpos = xPos + (int)(deltaX * 0.1f);
            int deltaY = 40;

            Color textsColor = Color.Yellow;
            Color valuesColor = Color.Red;

            string[] statsTexts = { "", "Zombies Killed", "Super Zombies Killed", "Zombie Cannons Killed", "Weapon Accuracy" };

            for (int i = 0; i < statsTexts.Length; i++)
            {
                spriteBatch.DrawString(spriteFont, statsTexts[i], new Vector2(startXpos, yPos + (i * deltaY)), textsColor);
            }

            startXpos += deltaX + 15;

            for (int i = 0; i < PlayerStats.Length; i++)
            {
                int y = yPos;
                spriteBatch.DrawString(spriteFont, "Player " + (i + 1), new Vector2(startXpos + (i * deltaX), y), textsColor);
                y += deltaY;
                spriteBatch.DrawString(spriteFont, PlayerStats[i].ZombieKills.ToString(), new Vector2(startXpos + (i * deltaX), y), valuesColor);
                y += deltaY;
                spriteBatch.DrawString(spriteFont, PlayerStats[i].SuperZombieKills.ToString(), new Vector2(startXpos + (i * deltaX), y), valuesColor);
                y += deltaY;
                spriteBatch.DrawString(spriteFont, PlayerStats[i].ZombieCannonKills.ToString(), new Vector2(startXpos + (i * deltaX), y), valuesColor);
                y += deltaY;

                float accuracy = ((float)PlayerStats[i].BulletsFired / ((float)PlayerStats[i].BulletsHit)) * 100;

                if (float.IsNaN(accuracy))
                {
                    accuracy = 0f;
                }

                spriteBatch.DrawString(spriteFont, accuracy.ToString("00.00") + "%", new Vector2(startXpos + (i * deltaX), y), valuesColor);
            }

            return yPos + (deltaY * statsTexts.Length);
           
        }
    }
}
