﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine.Xml;
using GameEngine;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using GameEngine.Animation;
using GameEngine.Physics;
using GameEngine.Helpers;
using Microsoft.Xna.Framework.Content;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework.Audio;

namespace GamePrototype.GameLogic
{
    public class ZombieBear : Enemy
    {
        private const int COOL_DOWN_TIME = 350;
        private const int ATTACK_REACH = 43;
        private const float ZOMBIE_RADIUS = 33f;

        private int reach = ATTACK_REACH;
        private float damage;
        
        private const string ZOMBIE_DEAD_SOUND = "Audio/zombiedead";
        public ModelNode Model { get; set; }
        private CollisionSimulator collisionSimulator;        
        private const int DEATH_ANIMATION_LENGTH = 4000;
        private float animationStarted;
        private bool animationDone = false;
        private Quaternion spinDeathRotation;
        private Vector3 spinDeathDirection;
        private float lastAttack = 0;
        private bool canAttack = true;

        Player attackTarget;
        public float superFactor = 1f;
        bool isAttacking = false;
        int attackFrame = 0;
        int dmgDealingFrame = 2;
        bool damageDealt = false;
        float attackTime = 0;
        public float[] KeyFrames = { 0.0f, 0.05f, 0.15f, 0.2f };
        public Quaternion[] Rotations = { Quaternion.Identity, Quaternion.CreateFromYawPitchRoll(-MathHelper.PiOver4 / 2f, 0, 0), Quaternion.CreateFromYawPitchRoll(MathHelper.PiOver4, 0, 0), Quaternion.Identity };
        public string[] AttackSounds = new string[] { "Audio/Ambient/bear4", "Audio/Ambient/bear3", "Audio/Ambient/bear1" };
        public static string[] SuperSounds = new string[] { "Audio/Arnies/Arni1of2", "Audio/Arnies/Arni2of2", "Audio/Arnies/ArniARRGE1", "Audio/Arnies/ArniARRGE2", "Audio/Arnies/ArniMonkey" };
        private GameWrapper gameWrapper;

        public bool IsProjectile = false;
        public float FlyingSpeed = 300f;
        public bool IsGrowing = false;
        private float hpToAddWhileGrowingPrSec = 0;
        float growPrSecond = 0.1f;

        public ZombieBear(string id, float damage, int hitpoints, float moveSpeed,CollisionSimulator collisionSimulator, ContentManager content, GameWrapper wrapper) : base(id, hitpoints, moveSpeed)
        {
            this.damage = damage;
            this.collisionSimulator = collisionSimulator;
            this.gameWrapper = wrapper;
            AudioManager.LoadSoundEffect(content, ZOMBIE_DEAD_SOUND);
            Model = new ModelNode("ZombieModel");
            Model.Asset = @"Models\zombiebear\deadbear";
            Model.AnimationFactor = 2f;
            Model.Position.X += 4;
            this.radius = ZOMBIE_RADIUS;
            this.ChildNodes.Add(Model);
        }

        public void DealDamage()
        {
            if (Dead || damageDealt)
            {
                damageDealt = true;
                return;
            }
            damageDealt = true;
            Vector2 me = new Vector2(this.WorldPosition.X, this.WorldPosition.Z), him = new Vector2(attackTarget.WorldPosition.X, attackTarget.WorldPosition.Z);
            BoundingCircle bc = new BoundingCircle(me, reach);
            BoundingCircle pc = new BoundingCircle(him, attackTarget.radius);
            Vector2 v;

            if (pc.Intersects(bc, out v))
            {
                attackTarget.TakeDamage((int)damage, Vector2.Zero);
                if (superFactor == 1.5f)
                {
                    AudioManager.PlaySoundEffect(SuperSounds[Util.Random.Next(SuperSounds.Length)], 1.5f, (float)Util.Random.NextDouble() * -0.25f);
                }
                else
                {
                    AudioManager.PlaySoundEffect(AttackSounds[Util.Random.Next(AttackSounds.Length)], 1f, (float)Util.Random.NextDouble() - 1.5f + superFactor);
                }
                gameWrapper.blood.DoFragments(me, Vector2.Normalize(him - me), 10, 50);
                canAttack = false;
                lastAttack = 0;
            }
        }

        public void UpdateAttackAnimation(GameTime time)
        {
            attackTime += time.ElapsedSeconds();
            if (attackTime >= KeyFrames[KeyFrames.Length - 1])
            {
                isAttacking = false;
                attackFrame = 0;
                attackTime = 0;
                if (!damageDealt)
                    DealDamage();
                damageDealt = false;
                return;
            }

            for (int i = 0; i < KeyFrames.Length; i++)
            {
                if (KeyFrames[i] <= attackTime)
                {
                    if (dmgDealingFrame == attackFrame && attackFrame != i)
                    {
                        DealDamage();
                    }
                    attackFrame = i;
                }
            }

            float d = KeyFrames[attackFrame + 1] - KeyFrames[attackFrame];
            float dt = attackTime - KeyFrames[attackFrame];

            Quaternion rotation = Quaternion.Slerp(Rotations[attackFrame], Rotations[attackFrame + 1], dt/d);
            this.Model.AnimationPlayer.BoneManipulators.Add(new BoneManipulator("Bip01_Spine1", Matrix.CreateFromQuaternion(rotation)));

        }

        public override void Update(GameTime gameTime, DrawInfo di)
        {

            if (IsProjectile)
            {
                moveDirection.Y = 0;
                Rotation = Quaternion.CreateFromAxisAngle(Vector3.Up, targetDirection + MathHelper.PiOver2) * Quaternion.CreateFromAxisAngle(Vector3.UnitX, MathHelper.PiOver2);
                Position += moveDirection * gameTime.ElapsedSeconds() * FlyingSpeed;
                Position.Y = 50f;
                if (RecentCollision)
                {
                    AudioManager.PlaySoundEffect("Audio/whack", 1.5f, (float)Util.Random.NextDouble() - 0.5f);

                    Rotation = Quaternion.CreateFromAxisAngle(Vector3.Up, targetDirection + MathHelper.PiOver2);
                    Position.Y = 0;
                    IsProjectile = false;
                    DisableTic = false;
                    SetNextWaypoint(CurrentTarget.Position);
                    currentDirection = targetDirection;
                    IsGrowing = true;
                }
            }
            else
            {
                if (!canAttack)
                {
                    lastAttack += gameTime.ElapsedMillis();

                    if (lastAttack > COOL_DOWN_TIME)
                    {
                        canAttack = true;
                    }
                }
                else
                {
                    Vector2 me = new Vector2(this.WorldPosition.X, this.WorldPosition.Z), him = new Vector2(CurrentTarget.WorldPosition.X, CurrentTarget.WorldPosition.Z);
                    BoundingCircle bc = new BoundingCircle(me, reach);
                    BoundingCircle pc = new BoundingCircle(him, CurrentTarget.radius);
                    Vector2 v;

                    if (pc.Intersects(bc, out v))
                    {
                        TryDealDamage(CurrentTarget);
                    }
                }

                List<BoneManipulator> manipulators = this.Model.AnimationPlayer.BoneManipulators;
                manipulators.Clear();

                if (isAttacking)
                    UpdateAttackAnimation(gameTime);

                if (hitpoints <= 0 && !Dead)
                {
                    Dead = true;
                    this.Model.AnimationPlayer.Pause = true;

                    if (superFactor == 1.5)
                    {
                        if (Util.Random.Next(100) >= 50)
                        {
                            Weapon item;
                            int rand = Util.Random.Next(10);
                            if (rand >= 8)
                                item = WeaponFactory.GetWeapon(WeaponType.Minigun);
                            else if (rand >= 4)
                                item = WeaponFactory.GetWeapon(WeaponType.Garand);
                            else
                                item = WeaponFactory.GetWeapon(WeaponType.M4);

                            gameWrapper.ThrowItemOnFloor(SpawnPointNode.SpawnType.BigWeapon, item, this);
                        }
                    } else if(superFactor > 1)
                        gameWrapper.ThrowItemOnFloor(SpawnPointNode.SpawnType.BigAmmo, "BigAmmo", this);

                    float speed = GenerateValues();
                    animationDone = false;
                    animationStarted = 0;
                    AudioManager.PlaySoundEffect(ZOMBIE_DEAD_SOUND, 0.4f, (speed - 0.5f) * 1.5f - 0.25f);
                    this.PassThrough = true;

                }

                if (Dead && !animationDone)
                {
                    animationStarted += gameTime.ElapsedMillis();

                    //List<BoneManipulator> manipulators = this.Model.AnimationPlayer.BoneManipulators;

                    /*
                    manipulators.Add(new BoneManipulator("Bip01_Spine2", Matrix.CreateRotationY(yRotation)));
                    manipulators.Add(new BoneManipulator("Bip01_Spine2", Matrix.CreateRotationZ(zRotation)));
                    */
                    /*manipulators.Add(new BoneManipulator("Bip01_Spine2", Matrix.CreateTranslation(
                                                                         new Vector3(xTranslation,
                                                                                     yTranslation,
                                                                                     zTranslation))));*/

                    Model.Rotation *= spinDeathRotation; //Quaternion.CreateFromYawPitchRoll(yRotation, xRotation, zRotation);
                    Model.Position += spinDeathDirection;
                    if (animationStarted > DEATH_ANIMATION_LENGTH)
                    {
                        animationDone = true;
                        Model.Position = new Vector3(4, 0, 0);
                        Model.Rotation = Quaternion.Identity;
                        gameWrapper.enemyController.DespawnEnemy(this);
                    }
                }
            }

            if (IsGrowing)
            {
                Grow(gameTime);
            }

            if(this.Parent != null)
                base.Update(gameTime, di);    
        }

        public float GenerateValues()
        {
            float speed = (float)Util.Random.NextDouble();

            Vector3 direction = new Vector3((float)Util.Random.NextDouble() - 0.5f, (float)Util.Random.NextDouble() - 0.5f, 0);
            Vector3 rotation = new Vector3((float)Util.Random.NextDouble() - 0.5f, (float)Util.Random.NextDouble() - 0.5f, (float)Util.Random.NextDouble() - 0.5f);

            direction.Y = Math.Abs(direction.X + direction.Z + direction.X + direction.Z);

            if(direction != Vector3.Zero)
                direction.Normalize();
    
            if (rotation != Vector3.Zero)
                rotation.Normalize();

            direction *= 10f + speed * 25f;
            rotation *= 0.05f + speed / 4f;

            spinDeathDirection = direction;
            spinDeathRotation = Quaternion.CreateFromYawPitchRoll(rotation.Y, rotation.X, rotation.Z);

            /*
            /// WHAT THE FUCK?!??! 
            int extraBonus = 0;

            if (xRotation + zRotation > 0.6f)
            {
                extraBonus += 5;
            }

            xTranslation = random.Next(10);
            zTranslation = random.Next(10);
            yTranslation = xTranslation + zTranslation + 1 + extraBonus;
            */
            return speed;
        }

        public override void Reset()
        {
            if(this.Model.AnimationPlayer != null)
                this.Model.AnimationPlayer.Pause = false;
            Model.Rotation = Quaternion.Identity;
            Dead = false;
            Position.Y = 0;
            superFactor = 1f;
            IsGrowing = false;
            SetSize(superFactor);
            base.Reset();
        }

        public static void FixZombieModel(Model model)
        {
            AnimationData data = ((Dictionary<string, object>)model.Tag)["AnimationData"] as AnimationData;
            foreach (var keyframe in data.Tracks["TwoSteps"].Keyframes)
            {
                if (keyframe.Bone != 0)
                    continue;

                Matrix t = keyframe.Transform;
                t.Translation = new Vector3(0, t.Translation.Y, 0);
                keyframe.Transform = t;
            }

            model.Meshes["mesh_Eye_Left01"].Effects[0].Parameters["specularIntensity"].SetValue(1f);
            model.Meshes["mesh_Eye_Right"].Effects[0].Parameters["specularIntensity"].SetValue(1f);
            model.Meshes["mesh_Axe_Head"].Effects[0].Parameters["specularIntensity"].SetValue(1f);
            model.Meshes["mesh_SkinnedMesh_Zbear"].Effects[0].Parameters["specularIntensity"].SetValue(0.3f);

        }

        public void TryDealDamage(Player target)
        {
            attackTarget = target;

            if (!isAttacking && canAttack)
            {
                isAttacking = true;
            }
        }

        /// <summary>
        /// Only call this AFTER model has been loaded! 
        /// </summary>
        public void SetSuper(float superFactor)
        {
            this.superFactor = superFactor;
            SetSize(superFactor);

            // ong...
            hitpoints = (int)((float)gameWrapper.enemyController.HITPOINTS[gameWrapper.enemyController.difficulty - 1] * superFactor);
            
            hpToAddWhileGrowingPrSec = ((float)gameWrapper.enemyController.HITPOINTS[gameWrapper.enemyController.difficulty - 1] * 1.5f) - hitpoints;
            hpToAddWhileGrowingPrSec *= growPrSecond / (1.5f - superFactor);
        }
        
        private void Grow(GameTime gt)
        {
            float dtSize = growPrSecond * gt.ElapsedSeconds();
            
            superFactor = Math.Min(superFactor + dtSize, 1.5f);
            if (superFactor == 1.5f)
            {
                IsGrowing = false;
            }
            hitpoints += hpToAddWhileGrowingPrSec * gt.ElapsedSeconds();
            SetSize(superFactor);


        }

        private void SetSize(float superFactor)
        {

            if (superFactor >= 1)
            {
                Vector3 color = Vector3.One - new Vector3(0.20f, 0.125f, 0.05f);
                color *= (superFactor - 1f) * 2f;
                Model.tintColor = new Vector4(Vector3.One - color, 1f);
            }
            else
            {
                Vector3 color = new Vector3(1f, 1f, 0f);
                color += (Vector3.One - color) * (superFactor - 0.4f) * 1.66666666666f;
                Model.tintColor = new Vector4(color, 1f);
            }
            Model.Scale = new Vector3(superFactor);
            radius = ZOMBIE_RADIUS * superFactor;
            reach = (int)(ATTACK_REACH * superFactor);
            moveSpeed = gameWrapper.enemyController.GetMovementSpeed(gameWrapper.enemyController.difficulty) * superFactor;
            damage = gameWrapper.enemyController.DAMAGE[gameWrapper.enemyController.difficulty - 1] * superFactor;


        }

    }
}
