﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework;
using GameEngine.Xml;
using Microsoft.Xna.Framework.Graphics;
using GameEngine.Helpers;
using GameEngine.LevelGeneration;
using GameEngine.Cameras;
using System.Xml;
using Microsoft.Xna.Framework.Content;
using GamePrototype.GameLogic;

namespace GamePrototype
{
    public class LevelRenderer
    {
        private LevelGenerator levelGenerator;

        public LevelTile[,] Level;
        private List<LevelTile> tiles;

        public Point LevelOffset { get; private set; }
        public static int TilesRendered = 0;
        public LevelRenderer()
        {
            tiles = new List<LevelTile>();
        }

        public void Initialize(GraphicsDevice graphics, ContentManager content, LevelGenerator levelGenerator)
        {
            this.levelGenerator = levelGenerator;


            
            int levelWidth, levelHeight;
            Level = levelGenerator.GetLevelTileArray(out levelWidth, out levelHeight, 1);
            for (int x = 0; x < Level.GetLength(0); x++)
            {
                for (int y = 0; y < Level.GetLength(1); y++)
                {
                    if (Level[x, y] != null && Level[x, y].X == 0 && Level[x, y].Y == 0)
                    {
                        LevelOffset = new Point(x, y);
                    }
                }
            }
            
            for (int x = 0; x < Level.GetLength(0); x++)
            {
                for (int y = 0; y < Level.GetLength(1); y++)
                {
                    if (Level[x, y] == null)
                    {
                        if ((x - 1 < 0 || Level[x - 1, y] == null || IsEmptyTile(Level[x - 1, y]))
                            && (x + 1 >= Level.GetLength(0) || Level[x + 1, y] == null || IsEmptyTile(Level[x + 1, y]))
                            && (y - 1 < 0 || Level[x, y - 1] == null || IsEmptyTile(Level[x, y - 1]))
                            && (y + 1 >= Level.GetLength(1) || Level[x, y + 1] == null || IsEmptyTile(Level[x, y + 1])))
                        {
                            continue;
                        }
                        else
                        {   
                            // This tile is empty, we want to draw som BLACKNESS there as to obscure other tiles' external faces.

                            Level[x, y] = new LevelTile(x - LevelOffset.X, y - LevelOffset.Y);
                            LevelTileNode ltn = new LevelTileNode("leveltile-(" + x + "," + y + ")");
                            Level[x, y].SceneNode = ltn;
                            ltn.IsEmpty = true;
                            ltn.Size = TilesetManager.CurrentTileset.TileSize;
                            ltn.Height = TilesetManager.CurrentTileset.Height;
                            ltn.Tile = Level[x, y];
                            ltn.LoadContent(content, graphics);
                            tiles.Add(Level[x, y]);
                        }
                    }
                    else
                    {
                        LevelTileNode ltn = new LevelTileNode("leveltile-(" + x + "," + y + ")");
                        if(Level[x,y].SpecialType == SpecialTileNodeType.None)
                            ltn.ChildNodes.AddRange(TilesetManager.LoadRandomTile(Level[x, y].TileType));
                        else
                            ltn.ChildNodes.AddRange(TilesetManager.LoadRandomSpecialTile(Level[x, y].SpecialType));

                        Level[x, y].SceneNode = ltn;
                        ltn.Size = TilesetManager.CurrentTileset.TileSize;
                        ltn.Height = TilesetManager.CurrentTileset.Height;
                        ltn.Tile = Level[x, y];
                        ltn.LoadContent(content, graphics);
                        tiles.Add(Level[x, y]);
                    }
                }
            }
        }

        public bool IsEmptyTile(LevelTile tile)
        {
            return tile != null && tile.SceneNode != null && ((LevelTileNode)tile.SceneNode).IsEmpty;
        }

        public void Update(GameTime gameTime, DrawInfo di)
        {
            for (int i = 0; i < tiles.Count; i++)
            {
                tiles[i].SceneNode.Update(gameTime, di);
                tiles[i].SceneNode.ChildNodes.UpdateRecursive(gameTime, di);
            }
        }

        public void Draw(GameTime gameTime, DrawInfo di, Point start)
        {
            Point tileCoords;
            int x = Math.Max(start.X - GameWrapper.WORLD_MAX_COLLIDE_DIST, 0);
            int ex = Math.Min(start.X + GameWrapper.WORLD_MAX_COLLIDE_DIST, Level.GetLength(0) - 1);

            int y = Math.Max(start.Y - GameWrapper.WORLD_MAX_COLLIDE_DIST, 0);
            int ey = Math.Min(start.Y + GameWrapper.WORLD_MAX_COLLIDE_DIST, Level.GetLength(1) - 1);

            for (tileCoords.X = x; tileCoords.X <= ex; tileCoords.X++)
            {
                for (tileCoords.Y = y; tileCoords.Y <= ey; tileCoords.Y++)
                {
                    LevelTile tile = Level[tileCoords.X, tileCoords.Y];
                    if (tile != null)
                    {

                    }
                }
            }

        }
    }
}
