﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework.Input;
using GameEngine.Xml;
using GameEngine.Helpers;
using Microsoft.Xna.Framework.Graphics;

namespace GamePrototype.Levels
{
    public class Circlecollision : BaseLevel
    {
        PolygonDrawer debugDrawer;
        PrimitivesDrawer primitivesDrawer;
        BoundingCircle testCirle = new BoundingCircle(new Vector2(0, 250), 10f);
        CollisionRay testRay = new CollisionRay(Vector2.Zero, Vector2.One);

        bool collided = false;


        public Circlecollision(Game game, GraphicsDeviceManager graphics, string asset)
            : base(game, graphics, asset)
        {
            Camera.FieldOfView = MathHelper.PiOver4 + MathHelper.PiOver4 / 2f;
            Camera.Position = new Vector3(0, 250f, 250f);
            Camera.Target = new Vector3(0, 0, 0f);
            testCirle.Center = new Vector2(0, 250);
            testRay.SetAndNormalizeDirection(new Vector2(-1, -1));
        }

        public override void LoadContent()
        {
            base.LoadContent();
            debugDrawer = new PolygonDrawer(Content.Load<Effect>("hud/primitives"), Graphics.GraphicsDevice);
            primitivesDrawer = new PrimitivesDrawer(Content.Load<Effect>("hud/primitives"), Graphics.GraphicsDevice);

        }

        public override void Draw(GameTime dt)
        {
            base.Draw(dt);
            debugDrawer.DrawPolys(Forest, DrawInfo);

            primitivesDrawer.DrawCircle(testCirle, collided ? Color.Green : Color.Blue, DrawInfo);
            primitivesDrawer.DrawRay(testRay, rayCollided ? Color.Red : Color.Yellow, DrawInfo);
            primitivesDrawer.DrawCircle(colCircle, circleCircleCollision ? Color.Green : Color.Red, DrawInfo);
            primitivesDrawer.DrawRay(otherRay, otherRayCollision ? Color.Red : Color.Green, DrawInfo);
        }

        public void Draw2D(GameTime dt, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            Vector2 collisionPoint = testRay.Position + testRay.Direction * collisionDist;
            spriteBatch.DrawString(spriteFont, "Ray collided with: " + rayCollidedWith + " " + collisionDist.ToString() + " " + collisionPoint.ToString(), new Vector2(20, 20), Color.Red);
            spriteBatch.DrawString(spriteFont, "CollisionCount: " + collisionCount.ToString(), new Vector2(20, 40), Color.Red);
            Vector2 rayColPoint = testRay.Position + outDist * testRay.Direction;
            spriteBatch.DrawString(spriteFont, "RayCollisionDist: " + outDist.ToString() + ", point: " + rayColPoint.ToString(), new Vector2(20, 60), Color.Red);
            spriteBatch.DrawString(spriteFont, "CC ColPoint: " + ccColPoint, new Vector2(20, 80), Color.Red);
            spriteBatch.DrawString(spriteFont, "OtherRay: " + otherRayColPoint, new Vector2(20, 100), Color.Red);
        }


        Vector2 lastPosition = Vector2.Zero;
        float collisionDist = 0;
        string rayCollidedWith = "";
        bool rayCollided = false;

        bool circleCircleCollision = false;
        Vector2 ccColPoint = Vector2.Zero;

        BoundingCircle colCircle = new BoundingCircle(new Vector2(0, -50), 20);
        int collisionCount = 0;

        CollisionRay otherRay = new CollisionRay(new Vector2(0, -200), Vector2.UnitX);
        bool otherRayCollision = false;
        Vector2 otherRayColPoint = Vector2.Zero;

        float outDist = 0;

        public override void Update(GameTime dt)
        {
            // Handle input
            HandleInput(dt);
            base.Update(dt);

            testRay.Position = testCirle.Center;

            rayCollided = testRay.Intersects(colCircle, out outDist);

            circleCircleCollision = colCircle.Intersects(testCirle, out ccColPoint);

            float tmpRayColDist;
            if (testRay.Intersects(otherRay, out tmpRayColDist))
            {
                otherRayCollision = true;
                otherRayColPoint = testRay.Position + testRay.Direction * tmpRayColDist;
            }
            else
            {
                otherRayCollision = false;
            }

            Vector2 lastResolve = Vector2.Zero;
            collisionDist = float.MaxValue;
            rayCollidedWith = "";
            collisionCount = 0;
            foreach (var node in Forest.NodesDepthFirst)
            {
                PolygonNode poly = node as PolygonNode;
                if (poly == null)
                    continue;

                poly.Update(dt);

                Vector2 resolve;
                if (poly.Intersects(testCirle, out resolve))
                {
                    if (Vector2.Dot(lastResolve, resolve) < -0.00001f)
                    {
                        Vector2 oldDirection = new Vector2(-lastResolve.Y, lastResolve.X);
                        resolve += oldDirection * resolve.LengthSquared() / Vector2.Dot(resolve, oldDirection);
                    }
                    testCirle.Center += resolve;
                    lastResolve = resolve;
                }
                float tmpDist;
                if(poly.Intersects(testRay, out tmpDist)) 
                {
                    if (tmpDist < collisionDist)
                    {
                        collisionDist = tmpDist;
                        rayCollidedWith = poly.Name;
                    }

                    collisionCount++;
                }
            }

            Vector2 newPos = testCirle.Center;
            if (Math.Abs(testCirle.Center.X - lastPosition.X) < dt.ElapsedGameTime.TotalSeconds * 10) 
            {
                newPos.X = lastPosition.X;
            }
            if (Math.Abs(testCirle.Center.Y - lastPosition.Y) < dt.ElapsedGameTime.TotalSeconds * 10)
            {
                newPos.Y = lastPosition.Y;
            }
            testCirle.Center = newPos;

            lastPosition = testCirle.Center;
        }

        private void HandleInput(GameTime dt)
        {
            float dtSeconds = (float)dt.ElapsedGameTime.TotalSeconds;
            if (InputState.IsKeyDown(Keys.PageDown))
                Camera.Zoom(-dtSeconds * 1000f);
            if (InputState.IsKeyDown(Keys.PageUp))
                Camera.Zoom(dtSeconds * 1000f);

            Vector2 circleMovement = Vector2.Zero;
            if (InputState.IsKeyDown(Keys.Left))
                circleMovement.X -= dtSeconds * 100f;
            if (InputState.IsKeyDown(Keys.Right))
                circleMovement.X += dtSeconds * 100f;
            if (InputState.IsKeyDown(Keys.Up))
                circleMovement.Y -= dtSeconds * 100f;
            if (InputState.IsKeyDown(Keys.Down))
                circleMovement.Y += dtSeconds * 100f;

            if (InputState.IsKeyDown(Keys.A))
                testRay.Rotate(1f * dtSeconds);
            if (InputState.IsKeyDown(Keys.D))
                testRay.Rotate(-1f * dtSeconds);


            testCirle.Center += circleMovement;
        }
    }
}
