using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using System.Xml;
using GameEngine.Xml;
using GameEngine.LevelGeneration;
using GameEngine.Cameras;
using GameEngine;
using GamePrototype;
using GameEngine.ScreenSystem;
using GameEngine.Helpers;
using System.Diagnostics;
using GameEngine.Physics;

namespace Platformer
{
    public class Game1 : Game
    {
        private const int DEFAULT_WIDTH = 1024;
        private const int DEFAULT_HEIGHT = 768;

        private const string PLAYER_MODEL = @"Model_Groenting_highpoly/Groenting_highpoly";
        
        private GraphicsDeviceManager graphics;
        private SpriteBatch spriteBatch;
        private SpriteFont spriteFont;
        private XmlDocument level1Doc;
        private NodeList level1Scene;
        private LevelTile tile;
        private LevelTileNode level1Tile;
        private Player player;
        private OrthoOrbitCamera camera;
        private DrawInfo di = new DrawInfo();
        private DeferredRenderer deferredRenderer;
        private GeometryDrawer geomDrawer = new GeometryDrawer();
        private CollisionSimulator collisionSimulator;
        public static StarThrowerManager starThrowerManager;
        private NewYearBearMasterMind masterMind;
        private Texture2D texture;
        private bool playerLanded = false; 
        private byte fadeValue = 0;
        private Color fadeColor = Color.Black;
        private bool gameEnded = false;
        private bool gameWon = false;
        private bool playerIsHit = false;

        private SpawnPointNode endPointNode;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            graphics.PreparingDeviceSettings += new EventHandler<PreparingDeviceSettingsEventArgs>(graphics_PreparingDeviceSettings);
            graphics.IsFullScreen = true;
            graphics.SynchronizeWithVerticalRetrace = true;
            this.Window.Title = "The Platformer Game";
            this.IsFixedTimeStep = true;
        }

        void graphics_PreparingDeviceSettings(object sender, PreparingDeviceSettingsEventArgs e)
        {
            DisplayMode displayMode = GraphicsAdapter.DefaultAdapter.CurrentDisplayMode;
            e.GraphicsDeviceInformation.PresentationParameters.BackBufferFormat = displayMode.Format;
            e.GraphicsDeviceInformation.PresentationParameters.BackBufferWidth = graphics.IsFullScreen ? displayMode.Width : DEFAULT_WIDTH;
            e.GraphicsDeviceInformation.PresentationParameters.BackBufferHeight = graphics.IsFullScreen ? displayMode.Height : DEFAULT_HEIGHT;
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        protected override void LoadContent()
        {
            AudioManager.LoadSong(@"Sound\Party", Content);
            AudioManager.PlayMusic(@"Sound\Party");
            AudioManager.LoadSoundEffect(@"Sound\Boing-One Low", Content);

            spriteBatch = new SpriteBatch(GraphicsDevice);
            spriteFont = Content.Load<SpriteFont>("Arial");

            level1Doc = Content.Load<XmlDocument>("Maps\\test1");
            level1Scene = XmlTest.LoadScene(level1Doc);

            level1Tile = new LevelTileNode("root");

            texture = new Texture2D(GraphicsDevice, 1, 1, 1, TextureUsage.None, SurfaceFormat.Color);
            texture.SetData<Color>(new Color[] { Color.White });

            tile = new LevelTile(0, 0);
            tile.BottomEdge = tile.LeftEdge = tile.TopEdge = tile.RightEdge = EdgeState.Blocked;
            tile.SceneNode = level1Tile;
            level1Tile.Tile = tile;
            level1Tile.ChildNodes.AddRange(level1Scene);
            level1Tile.LoadContent(Content, GraphicsDevice);

            ModelNode mn = new ModelNode("model");
            mn.Asset = PLAYER_MODEL;
            mn.Scale = new Vector3(0.25f, 0.25f, 0.25f);
            mn.Position = new Vector3(0, -15, 0);
            mn.LoadContent(Content, GraphicsDevice);

            player = new Player("player");
            player.Position = new Vector3(0, 0, -100);
            player.radius = 14;
            player.ChildNodes.Add(mn);
            player.Rotation = Quaternion.CreateFromYawPitchRoll(MathHelper.PiOver2, 0, -MathHelper.PiOver2);
          
            level1Tile.AddDynamicNode(player);
            
            camera = new OrthoOrbitCamera(MathHelper.PiOver4, Vector3.UnitY, Vector3.Zero, -Vector3.UnitZ, GraphicsDevice.Viewport, 1, 10000, CameraMode.Perspective);

            di.GraphicsDevice = GraphicsDevice;
            di.Camera = camera;
            di.AllCameras = new Camera[1] { camera };
            
            deferredRenderer = new DeferredRenderer(GraphicsDevice.Viewport.Width, GraphicsDevice.Viewport.Height);
            //deferredRenderer.TargetEndResult = DeferredRenderer.TargetOutput.Color;
            deferredRenderer.LoadContent(GraphicsDevice, Content);
            //deferredRenderer.SetBGColor(Color.Black);
            
            geomDrawer.Load(Content, GraphicsDevice);
            
            starThrowerManager = new StarThrowerManager();
            starThrowerManager.Load(Content, GraphicsDevice);

            masterMind = new NewYearBearMasterMind(player, level1Tile, Content, GraphicsDevice);
                        
            LevelTile[,] tileArray = new LevelTile[1, 1];
            tileArray[0,0] = tile;
            collisionSimulator = new CollisionSimulator(tileArray, 10000);
            collisionSimulator.OnCollision += new OnCollision(collisionSimulator_OnCollision);


            n = new PointLightNode("SomeLight");
            n.LightIntensity = 0.8f;
            n.Position = new Vector3(100, -500, -100);
            n.LightRadius = 10000;

            endPointNode = tile.SceneNode.GetGoodSpawnPoint(SpawnPointNode.SpawnType.Health);
            endPointNode.radius = 25;

        }
        PointLightNode n;
        void collisionSimulator_OnCollision(ObjectNode a, ObjectNode b)
        {
            if (a is Player && b is PolygonWallNode)
            {
                if ((a as DynamicCircleNode).oldResolve.Y < 0)
                {
                    playerLanded = true;
                }
            }

            if (a is NewYearBear && b is NewYearBear)
            {
                (a as NewYearBear).Direction *= -1;
                (b as NewYearBear).Direction *= -1;
            }

            if (a is Player && b is NewYearBear)
            {
                playerIsHit = true;
            }

            if (a is Player && b is SpawnPointNode && (b as SpawnPointNode).SpawnPointType == SpawnPointNode.SpawnType.Health)
            {
                gameWon = true;
            }
        }

        protected override void UnloadContent()
        {
        
        }

        protected override void Update(GameTime gameTime)
        {
            InputState.Update();
            n.Update(gameTime, di);
            masterMind.Update(gameTime);
            player.airborne = !playerLanded;
            playerLanded = false;
            
            starThrowerManager.Update(gameTime);

            level1Tile.Update(gameTime, di);
            level1Tile.ChildNodes.UpdateRecursive(gameTime, di);

            player.Update(gameTime);
            
            collisionSimulator.Update(gameTime, new Point[] { Point.Zero }, 0);


            if (InputState.KeyPressed(Keys.Escape)) { this.Exit(); }
            
            if (InputState.KeyboardState.IsKeyDown(Keys.Up))
            {
                player.Jump();
            }

            if (InputState.KeyboardState.IsKeyDown(Keys.Left))
            {
                player.MoveLeft(gameTime);
                player.Rotation = Quaternion.CreateFromYawPitchRoll(MathHelper.PiOver2, MathHelper.Pi, -MathHelper.PiOver2);
            }
            if (InputState.KeyboardState.IsKeyDown(Keys.Right))
            {
                player.MoveRight(gameTime);
                player.Rotation = Quaternion.CreateFromYawPitchRoll(MathHelper.PiOver2, 0, -MathHelper.PiOver2);
            }
                   
            camera.Position = player.Position + new Vector3(300, 600, -50);
            camera.Target = player.Position;
            camera.Target.Z -= 100;
            camera.UpdateCamera(gameTime);
            n.Position = camera.Position;
            if (fadeValue > 240)
            {
                gameEnded = true;
            }

            if (playerIsHit || gameWon)
            {
                fadeValue += 8;
            }
            
            if (player.WorldPosition.Z > 150)
            {
                fadeValue += 4;
            }
            
            if (InputState.KeyPressed(Keys.Enter) && gameEnded)
            {
                player.Position = new Vector3(0, 0, -100);
                starThrowerManager.StarThrowers.Clear();
                //LoadContent();
                masterMind.Reset();
                gameEnded = false;
                gameWon = false;
                playerIsHit = false;
                fadeValue = 0;
            }
            
            base.Update(gameTime);
        }
                
        protected override void Draw(GameTime gameTime)
        {
            if (!gameEnded)
            {
                di.PointLights.Add(n);

                deferredRenderer.BeginRender(di);
                level1Tile.Draw(gameTime, di);
                geomDrawer.Draw(di);
                starThrowerManager.Draw(di);

                deferredRenderer.EndRender(null);
                di.PointLights.Clear();
                spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);

                fadeColor.A = fadeValue;
                spriteBatch.Draw(texture, new Rectangle(0, 0, Window.ClientBounds.Width, Window.ClientBounds.Height), fadeColor);

                spriteBatch.End();
            }
            else
            {
                GraphicsDevice.Clear(Color.Black);

                spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
                string endMessage = gameWon ? "Game Won! Press Enter to try again..." : "Game Lost! Press Enter to try again...";
                spriteBatch.DrawString(spriteFont, endMessage, 
                    new Vector2(Window.ClientBounds.Width / 2 - 290, Window.ClientBounds.Height / 2 - 20), Color.Red, 0, Vector2.Zero, 1.8f, SpriteEffects.None, 0);
                
                spriteBatch.End();

            }
            
            base.Draw(gameTime);
        }
    }
}
