﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using GameEngine;
using GameEngine.Xml;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace Platformer
{
    public class NewYearBearMasterMind
    {
        private const string ZOMBIE_MODEL = @"Model_Zbearonsegway/zbearnsegway";
        
        private const int MIN_MS_BETWEEN_SPAWNS = 500;
        private const int MAX_MS_BETWEEN_SPAWNS = 1500;

        public List<NewYearBear> NewYearBears { get; set; }

        private LevelTileNode levelTileNode;
        private ContentManager content;
        private GraphicsDevice graphics;
        private Player player;
        private float timeSinceLastSpawn = 0;
        private float timeTillNextSpawn = 0;

        public NewYearBearMasterMind(Player player, LevelTileNode levelTileNode, ContentManager content, GraphicsDevice graphics)
        {
            this.levelTileNode = levelTileNode;
            this.player = player;
            this.content = content;
            this.graphics = graphics;
            this.NewYearBears = new List<NewYearBear>();
        }

        public void Update(GameTime gameTime)
        {
            for (int i = 0; i < NewYearBears.Count; i++)
            {
                NewYearBears[i].Update(gameTime);
                Matrix m = (NewYearBears[i].ChildNodes["model"] as ModelNode).Model.Meshes["mesh_MongolArmDimsForStarStuffing"].ParentBone.Parent.Transform;
                Matrix starThrowerPosition = m * Matrix.CreateScale((NewYearBears[i].ChildNodes["model"] as ModelNode).Scale) * NewYearBears[i].World;
                starThrowerPosition *= Matrix.CreateTranslation(NewYearBears[i].Direction < 1 ? -20 : 20, 0, 5);

                NewYearBears[i].StarThrower.Position = starThrowerPosition.Translation;

                if (NewYearBears[i].WorldPosition.Z > 300)
                {
                    Game1.starThrowerManager.StarThrowers.Remove(NewYearBears[i].StarThrower);
                    levelTileNode.RemoveDynamicNode(NewYearBears[i]);
                }
            }

            timeSinceLastSpawn += gameTime.ElapsedGameTime.Milliseconds;

            if (timeSinceLastSpawn > timeTillNextSpawn)
            {
                SpawnEnemy();

                timeSinceLastSpawn = 0;
                timeTillNextSpawn = Util.Random.Next(MIN_MS_BETWEEN_SPAWNS, MAX_MS_BETWEEN_SPAWNS);
            }                                   
        }

        private void SpawnEnemy()
        {
            Vector3 bearPosition = player.WorldPosition;
            bearPosition.Z -= 1200;
            bearPosition.X += Util.Random.Next(400) - 200;
            
            ModelNode mn = new ModelNode("model");
            mn.Asset = ZOMBIE_MODEL;
            mn.Scale = new Vector3(0.25f, 0.25f, 0.25f);
            mn.Position = new Vector3(0, -5, 0);
            mn.LoadContent(content, graphics);

            NewYearBear bear = new NewYearBear("bear");
            bear.Position = bearPosition;
            bear.radius = 18;
            bear.ChildNodes.Add(mn);

            Game1.starThrowerManager.StarThrowers.Add(bear.StarThrower);
            levelTileNode.AddDynamicNode(bear);
            NewYearBears.Add(bear);
        }
        public void Reset()
        {
            foreach (var item in NewYearBears)
            {
                levelTileNode.RemoveDynamicNode(item);
            }
            NewYearBears.Clear();
        }
    }
}
