﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine.Xml;
using Microsoft.Xna.Framework;
using GameEngine.ScreenSystem;

namespace Platformer
{
    public class Player : DynamicCircleNode
    {
        private const float GRAVITY = 0.1f;
        private const float MOVE_SPEED = 0.3f;
        private const float JUMP_TIMESPAN = 80;
        private const float JUMP_SPEED = 20.0f;
        private float jumpStart = 0;
        private bool isJumping = false;
        private Vector2 Force = Vector2.Zero;
        public bool airborne = false;
        
        public Player(string name) : base(name)
        {

        }

        public void Update(GameTime gameTime)
        {
            if (isJumping)
            {
                Force.Y -= JUMP_SPEED;

                jumpStart += gameTime.ElapsedGameTime.Milliseconds;

                if (jumpStart > JUMP_TIMESPAN)
                {
                    isJumping = false;
                }
            }

            Force.Y += gameTime.ElapsedGameTime.Milliseconds * GRAVITY;
            Force.Y = MathHelper.Clamp(Force.Y, -20f, 10f);
            Force.X = MathHelper.Clamp(Force.X, -3f, 3f);

            Position.X += Force.X;
            Position.Z += Force.Y;
            Force.X = 0;
        }

        public void Jump()
        {
            if (!airborne)
            {
                AudioManager.PlaySoundEffect(@"Sound\Boing-One Low", 1f);
                airborne = true;
                isJumping = true;
                jumpStart = 0;
            }
        }

        public void MoveLeft(GameTime gameTime)
        {
            Force.X -= MOVE_SPEED * gameTime.ElapsedGameTime.Milliseconds;
        }

        public void MoveRight(GameTime gameTime)
        {
            Force.X += MOVE_SPEED * gameTime.ElapsedGameTime.Milliseconds;
        }

        public void Squeeze(Vector2 squeezeVector)
        {
            
        }
    }
}
