using System;
using System.IO;
using System.Reflection;
using System.Collections.Generic;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;


namespace XNBContentCompiler
{
    public class XNBCompiler : IDisposable
    {
        
        ContentBuilder contentBuilder;
        ContentManager contentManager;
        CompilerDevice compilerDevice;

        public XNBCompiler(string[] PipelineAssemblies)
        {
            compilerDevice = new CompilerDevice();
            
            contentBuilder = new ContentBuilder(PipelineAssemblies);

            contentManager = new ContentManager(compilerDevice.Services,
                                                contentBuilder.OutputDirectory);
        }

        public XNBCompiler(IServiceProvider services, params ContentBuilder.ProjectReference[] projectReferences)
        {
            //compilerDevice = new CompilerDevice();

            contentBuilder = new ContentBuilder(projectReferences);

            contentManager = new ContentManager(services,
                                                contentBuilder.OutputDirectory);
        }



        public T LoadContent<T>(string fileName, 
            string contentImporter, 
            string contentProcessor, 
            out string buildError)
        {
            buildError = null;
            string contentName = Path.GetFileNameWithoutExtension(fileName);
            object content = null;

            // Unload any existing model.
            //contentManager.Unload();

            // Tell the ContentBuilder what to build.
            contentBuilder.Clear();
            
            contentBuilder.Add(fileName, contentName, contentImporter, contentProcessor);

            // Build this new model data.
            buildError = contentBuilder.Build();

            if (string.IsNullOrEmpty(buildError))
            {
                // If the build succeeded, use the ContentManager to
                // load the temporary .xnb file that we just created.
                try
                {
                    content = contentManager.Load<T>(contentName);
                }
                catch (ArgumentException)
                {
                    content = null;
                }
            }

            return (T)content;
        }

        public void Dispose()
        {
            if (compilerDevice != null)
            {
                compilerDevice.Dispose();
            }
        }
    }
}
